from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    # 僅在型別檢查時匯入，避免循環依賴
    # Only imported during type checking to avoid circular imports
    from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor

from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QSyntaxHighlighter
from PySide6.QtGui import QTextCharFormat

# 匯入語法設定，包括關鍵字、規則、擴展設定
# Import syntax settings: keywords, rules, and extended settings
from je_editor.pyside_ui.code.syntax.syntax_setting import (
    syntax_word_setting_dict,
    syntax_rule_setting_dict,
    syntax_extend_setting_dict
)
from je_editor.utils.logging.loggin_instance import jeditor_logger


class PythonHighlighter(QSyntaxHighlighter):
    """
    Python 語法高亮類別，繼承自 QSyntaxHighlighter
    Python syntax highlighter class, inherits from QSyntaxHighlighter
    """

    def __init__(self, parent=None, main_window: CodeEditor = None):
        jeditor_logger.info(f"Init PythonHighlighter parent: {parent}")
        super().__init__(parent)

        self.highlight_rules = []  # 儲存所有高亮規則 / store all highlight rules

        # 判斷目前檔案副檔名，若無則預設為 .py
        # Determine current file suffix, default to .py
        if main_window is not None and main_window.current_file is not None:
            current_file_suffix = Path(main_window.current_file).suffix
        else:
            current_file_suffix = ".py"

        # -------------------------
        # 基本語法規則 (通用)
        # Basic highlight rules (common)
        # -------------------------
        for rule_variable_dict in syntax_rule_setting_dict.values():
            color = rule_variable_dict.get("color")  # 規則顏色 / rule color
            text_char_format = QTextCharFormat()
            text_char_format.setForeground(color)
            for rule in rule_variable_dict.get("rules"):  # 正則規則 / regex rules
                pattern = QRegularExpression(rule)
                self.highlight_rules.append((pattern, text_char_format))

        # -------------------------
        # Python 語法高亮
        # Python-specific highlight
        # -------------------------
        if current_file_suffix == ".py":
            for rule_variable_dict in syntax_word_setting_dict.values():
                color = rule_variable_dict.get("color")
                text_char_format = QTextCharFormat()
                text_char_format.setForeground(color)
                for word in rule_variable_dict.get("words"):  # 關鍵字清單 / keyword list
                    # 使用 \b 確保完整單字匹配 / use \b for whole word match
                    pattern = QRegularExpression(rf"\b{word}\b")
                    self.highlight_rules.append((pattern, text_char_format))

        # -------------------------
        # 其他語言的擴展高亮（透過插件系統）
        # Extended highlight for other languages (via plugin system)
        # -------------------------
        else:
            from je_editor.plugins import get_programming_language_plugin
            plugin = get_programming_language_plugin(current_file_suffix)
            if plugin:
                # 載入插件的額外語法規則（正則表達式）
                # Load plugin's extra syntax rules (regex patterns)
                for rule_variable_dict in plugin.get("syntax_rules", {}).values():
                    color = rule_variable_dict.get("color")
                    text_char_format = QTextCharFormat()
                    text_char_format.setForeground(color)
                    for rule in rule_variable_dict.get("rules"):
                        pattern = QRegularExpression(rule)
                        self.highlight_rules.append((pattern, text_char_format))

                # 載入插件的關鍵字高亮
                # Load plugin's keyword highlighting
                for rule_variable_dict in plugin.get("syntax_words", {}).values():
                    color = rule_variable_dict.get("color")
                    text_char_format = QTextCharFormat()
                    text_char_format.setForeground(color)
                    for word in rule_variable_dict.get("words"):
                        pattern = QRegularExpression(rf"\b{word}\b")
                        self.highlight_rules.append((pattern, text_char_format))
            elif syntax_extend_setting_dict.get(current_file_suffix):
                # 向後相容：使用舊的 syntax_extend_setting_dict
                # Backward compatible: use old syntax_extend_setting_dict
                for rule_variable_dict in syntax_extend_setting_dict.get(current_file_suffix).values():
                    color = rule_variable_dict.get("color")
                    text_char_format = QTextCharFormat()
                    text_char_format.setForeground(color)
                    for word in rule_variable_dict.get("words"):
                        pattern = QRegularExpression(rf"\b{word}\b")
                        self.highlight_rules.append((pattern, text_char_format))

    def highlightBlock(self, text) -> None:
        """
        對每一行文字進行語法高亮
        Apply syntax highlighting to each block of text
        """
        jeditor_logger.info(f"PythonHighlighter highlightBlock text: {text}")
        for pattern, pattern_format in self.highlight_rules:
            match_iterator = pattern.globalMatch(text)  # 全域比對 / global regex match
            while match_iterator.hasNext():
                match = match_iterator.next()
                # 設定比對到的文字格式 / apply format to matched text
                self.setFormat(match.capturedStart(), match.capturedLength(), pattern_format)



import os

from PySide6.QtCore import QObject, QProcess, Signal, QTimer


class ConsoleProcessAdapter(QObject):
    """
    ConsoleProcessAdapter 負責管理 QProcess，提供互動式 shell 的啟動、指令傳送與輸出監聽。
    ConsoleProcessAdapter manages QProcess, providing interactive shell start, command sending, and output handling.
    """

    # 定義訊號 / Define signals
    started = Signal()  # 當子程序啟動時發射 / Emitted when process starts
    finished = Signal(int, QProcess.ExitStatus)  # 當子程序結束時發射 / Emitted when process finishes
    stdout = Signal(str)  # 標準輸出訊號 / Standard output signal
    stderr = Signal(str)  # 錯誤輸出訊號 / Standard error signal
    system = Signal(str)  # 系統訊息訊號 / System message signal

    def __init__(self, parent=None):
        super().__init__(parent)
        # 建立 QProcess 物件 / Create QProcess object
        self.proc = QProcess(self)
        # 設定輸出通道分離 (stdout / stderr) / Separate stdout and stderr
        self.proc.setProcessChannelMode(QProcess.ProcessChannelMode.SeparateChannels)

        # 綁定事件處理函式 / Connect signals to handlers
        self.proc.readyReadStandardOutput.connect(self._on_stdout)
        self.proc.readyReadStandardError.connect(self._on_stderr)
        self.proc.started.connect(self.started)
        self.proc.finished.connect(self.finished)

    # 設定工作目錄 / Set working directory
    def set_cwd(self, path: str):
        self.proc.setWorkingDirectory(path)

    # 啟動 shell / Start shell
    def start_shell(self, shell: str = "auto"):
        if self.is_running():
            self.system.emit("Shell already running")  # 如果已經在執行，發送提示 / Emit message if already running
            return
        program, args = self._build_shell_command(shell)  # 建立 shell 指令 / Build shell command
        self.proc.start(program, args)  # 啟動子程序 / Start process

        # Windows 特殊處理：設定 UTF-8 編碼 / Windows-specific: set UTF-8 encoding
        if os.name == "nt":
            QTimer.singleShot(500, lambda: self.send_command("chcp 65001"))

    # 傳送指令到 shell / Send command to shell
    def send_command(self, cmd: str):
        if not self.is_running():
            self.system.emit("Shell not running")  # 如果 shell 未啟動，發送提示 / Emit message if not running
            return
        self.proc.write((cmd + "\n").encode("utf-8"))  # 傳送指令並換行 / Send command with newline

    # 停止 shell / Stop shell
    def stop(self):
        if not self.is_running():
            return
        self.proc.terminate()  # 嘗試正常結束 / Try graceful termination
        # 如果 1 秒後仍在執行，強制 kill / Force kill if still running after 1s
        QTimer.singleShot(1000, lambda: self.is_running() and self.proc.kill())

    # 判斷是否正在執行 / Check if process is running
    def is_running(self):
        return self.proc.state() != QProcess.ProcessState.NotRunning

    # 處理標準輸出 / Handle standard output
    def _on_stdout(self):
        self.stdout.emit(bytes(self.proc.readAllStandardOutput()).decode("utf-8", errors="replace"))

    # 處理錯誤輸出 / Handle standard error
    def _on_stderr(self):
        self.stderr.emit(bytes(self.proc.readAllStandardError()).decode("utf-8", errors="replace"))

    # 建立 shell 指令 / Build shell command
    def _build_shell_command(self, shell: str):
        if shell == "auto":
            shell = "cmd" if os.name == "nt" else "bash"  # Windows 預設 cmd，Linux/macOS 預設 bash
        if os.name == "nt":
            if shell == "powershell":
                return "powershell.exe", ["-NoLogo", "-NoProfile", "-ExecutionPolicy", "Bypass"]
            return "cmd.exe", []
        # Linux/macOS 預設 bash，否則使用 sh / Default bash, fallback to sh
        return ("/bin/bash" if shell == "bash" else "/bin/sh"), []



from __future__ import annotations

import os
from pathlib import Path
from typing import TYPE_CHECKING

from je_editor.pyside_ui.code.auto_save.auto_save_manager import file_is_open_manager_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    # 僅在型別檢查時匯入，避免循環依賴
    # Only imported during type checking to avoid circular imports
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

from PySide6.QtWidgets import QFileDialog

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.utils.file.save.save_file import write_file


def _build_save_file_filters() -> list[str]:
    """
    動態建立檔案儲存對話框的篩選器列表，包含 Python 與所有插件語言。
    Dynamically build file filter list for save dialog, including Python and all plugin languages.
    """
    from je_editor.plugins import get_all_plugin_run_configs

    # 預設篩選器 / Default filters
    filters = ["Python file (*.py)"]

    # 從插件執行設定收集篩選器 / Collect filters from plugin run configs
    for config in get_all_plugin_run_configs():
        name = config.get("name", "")
        suffixes = config.get("suffixes", ())
        if name and suffixes:
            suffix_pattern = " ".join(f"*{s}" for s in suffixes)
            filters.append(f"{name} ({suffix_pattern})")

    # 加入通用篩選器 / Add generic filters
    filters.append("HTML file (*.html)")
    filters.append("File (*.*)")

    return filters


def _select_filter_by_suffix(filters: list[str], suffix: str) -> str:
    """
    根據副檔名自動選擇對應的篩選器。
    Automatically select the matching filter based on file suffix.
    例如 suffix=".cpp" 會匹配含有 "*.cpp" 的篩選器。
    e.g. suffix=".cpp" matches the filter containing "*.cpp".
    """
    if not suffix:
        return filters[0]
    for f in filters:
        if f"*{suffix}" in f:
            return f
    return filters[0]


def choose_file_get_save_file_path(parent_qt_instance: EditorMain) -> bool:
    """
    開啟「另存新檔」對話框，將編輯器內容儲存到檔案
    Open "Save As" dialog and save editor content to file
    :param parent_qt_instance: Pyside 主視窗 / Pyside parent
    :return: 是否成功儲存檔案 / whether file was saved successfully
    """
    jeditor_logger.info("save_file_dialog.py choose_file_get_save_file_path"
                        f" parent_qt_instance: {parent_qt_instance}")

    # 取得目前分頁的編輯器元件
    # Get current tab's editor widget
    widget = parent_qt_instance.tab_widget.currentWidget()

    if isinstance(widget, EditorWidget):
        # 動態建立檔案篩選器（包含插件語言）
        # Dynamically build file filter (including plugin languages)
        filters = _build_save_file_filters()
        file_filter = ";;".join(filters)

        # 根據目前檔案的副檔名自動選擇對應篩選器
        # Auto-select filter based on current file's suffix
        current_suffix = ""
        if widget.current_file:
            current_suffix = Path(widget.current_file).suffix
        selected = _select_filter_by_suffix(filters, current_suffix)

        # 開啟檔案儲存對話框 / Open file save dialog
        file_path = QFileDialog().getSaveFileName(
            parent=parent_qt_instance,
            dir=os.getcwd(),
            filter=file_filter,
            selectedFilter=selected,
        )[0]

        # 確認使用者有選擇檔案路徑 / Ensure user selected a file path
        if file_path is not None and file_path != "":
            # 更新目前檔案路徑 / Update current file path
            widget.current_file = file_path

            # 將編輯器內容寫入檔案 / Write editor content to file
            write_file(file_path, widget.code_edit.toPlainText())

            # 更新已開啟檔案管理字典 / Update opened file manager dictionary
            path = Path(file_path)
            file_is_open_manager_dict.update({str(path): str(path.name)})

            # 若有自動儲存執行緒，更新其監控的檔案與編輯器
            # If auto-save thread exists, update its file and editor reference
            if widget.code_save_thread is not None:
                widget.code_save_thread.file = file_path
                widget.code_save_thread.editor = widget.code_edit

            # 更新分頁標題 / Update tab title
            widget.rename_self_tab()
            return True
        return False
    return False



from __future__ import annotations

import os
import queue
import shlex
import subprocess
import sys
from pathlib import Path
from threading import Thread
from typing import Union, Callable

from PySide6.QtCore import QTimer
from PySide6.QtGui import QTextCharFormat
from PySide6.QtWidgets import QTextEdit

from je_editor.pyside_ui.code.running_process_manager import run_instance_manager
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.utils.exception.exception_tags import je_editor_init_error
from je_editor.utils.exception.exceptions import JEditorException
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.venv_check.check_venv import check_and_choose_venv


class ShellManager(object):
    def __init__(
            self,
            main_window: Union[EditorWidget, None] = None,
            shell_encoding: str = "utf-8",
            program_buffer: int = 1024,
            after_done_function: Union[None, Callable] = None
    ):
        # 初始化 ShellManager，記錄基本設定
        # Initialize ShellManager, log basic settings
        jeditor_logger.info(f"Init ShellManager "
                            f"main_window: {main_window} "
                            f"shell_encoding: {shell_encoding} "
                            f"program_buffer: {program_buffer} "
                            f"after_done_function: {after_done_function}")
        """
        :param main_window: Pyside 主視窗 / Pyside main window
        :param shell_encoding: shell 輸出編碼 / shell command output encoding
        :param program_buffer: 緩衝區大小 / buffer size
        """
        self.read_program_error_output_from_thread = None  # 錯誤輸出讀取執行緒 / thread for reading stderr
        self.read_program_output_from_thread = None  # 標準輸出讀取執行緒 / thread for reading stdout
        self.main_window: EditorWidget = main_window  # 主視窗 / main window
        self.compiler_path = None  # Python 編譯器路徑 / Python compiler path
        self.code_result: Union[QTextEdit, None] = None  # 顯示輸出結果的文字框 / QTextEdit for displaying results
        self.timer: Union[QTimer, None] = None  # 定時器 / QTimer for periodic updates
        self.still_run_shell: bool = True  # 是否仍在執行 / flag for running state
        self.process = None  # 子程序物件 / subprocess object
        self.run_output_queue: queue.Queue = queue.Queue()  # 標準輸出佇列 / stdout queue
        self.run_error_queue: queue.Queue = queue.Queue()  # 錯誤輸出佇列 / stderr queue
        self.program_encoding: str = shell_encoding  # 編碼設定 / encoding setting
        self.program_buffer: int = program_buffer  # 緩衝區大小 / buffer size
        self.after_done_function = after_done_function  # 完成後的回呼函數 / callback after done
        self.renew_path()  # 更新 Python 執行路徑 / renew Python path
        run_instance_manager.instance_list.append(self)  # 註冊到執行管理器 / register instance

    def renew_path(self) -> None:
        # 更新 Python 編譯器路徑
        # Renew Python compiler path
        jeditor_logger.info("ShellManager renew_path")
        if self.main_window is None or self.main_window.python_compiler is None:
            # 如果主視窗沒有指定 Python，則使用 venv
            # If no compiler specified, use venv
            if sys.platform in ["win32", "cygwin", "msys"]:
                venv_path = Path(os.getcwd()) / "venv" / "Scripts"
            else:
                venv_path = Path(os.getcwd()) / "venv" / "bin"
            self.compiler_path = check_and_choose_venv(venv_path)
        else:
            self.compiler_path = self.main_window.python_compiler

    def later_init(self) -> None:
        # 延遲初始化，綁定輸出視窗
        # Late initialization, bind output QTextEdit
        jeditor_logger.info("ShellManager later_init")
        if self.main_window is not None:
            self.code_result: QTextEdit = self.main_window.code_result
        else:
            raise JEditorException(je_editor_init_error)

    def exec_shell(self, shell_command: Union[str, list]) -> None:
        """
        執行 shell 指令
        Execute shell command
        :param shell_command: 要執行的指令 / command to run
        :return: 若錯誤則回傳錯誤訊息 / return error message if failed
        """
        jeditor_logger.info(f"ShellManager exec_shell, shell_command: {shell_command}")
        try:
            self.exit_program()  # 結束舊的程序 / terminate previous process
            self.code_result.setPlainText("")  # 清空輸出視窗 / clear output window
            # shell=True expects a string on Unix, list on Windows is also fine
            if sys.platform in ["win32", "cygwin", "msys"]:
                args = shell_command
            else:
                args = shell_command if isinstance(shell_command, str) else " ".join(shell_command)
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText(str(args), text_format)  # 顯示執行的指令 / show executed command
            text_cursor.insertBlock()
            # 建立子程序 / create subprocess
            self.process = subprocess.Popen(
                args=args,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=True,
            )
            self.still_run_shell = True
            # 建立讀取標準輸出的執行緒 / thread for stdout
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            )
            self.read_program_output_from_thread.start()
            # 建立讀取錯誤輸出的執行緒 / thread for stderr
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            )
            self.read_program_error_output_from_thread.start()
            # 啟動定時器，每 10ms 更新一次輸出 / start QTimer for updating output
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(10)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()
        except Exception as error:
            # 若發生錯誤，顯示錯誤訊息並結束程序
            # If error occurs, show error message and terminate process
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("error_output_color"))
            text_cursor.insertText(str(error), text_format)
            text_cursor.insertBlock()
            if self.process is not None:
                self.process.terminate()

    # tkinter_ui update method
    def pull_text(self) -> None:
        # 從佇列中拉取輸出與錯誤訊息，並更新到 UI
        # Pull stdout/stderr messages from queue and update UI
        jeditor_logger.info("ShellManager pull_text")
        try:
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    text_cursor = self.code_result.textCursor()
                    text_format = QTextCharFormat()
                    text_format.setForeground(actually_color_dict.get("normal_output_color"))
                    text_cursor.insertText(output_message, text_format)
                    text_cursor.insertBlock()
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    text_cursor = self.code_result.textCursor()
                    text_format = QTextCharFormat()
                    text_format.setForeground(actually_color_dict.get("error_output_color"))
                    text_cursor.insertText(error_message, text_format)
                    text_cursor.insertBlock()
        except queue.Empty:
            pass
        # 檢查子程序是否結束 / check if process finished
        if self.process is not None:
            if self.process.returncode == 0:
                self.process_run_over()
            elif self.process.returncode is not None:
                self.process_run_over()
            if self.still_run_shell:
                # 持續檢查程序狀態 / keep polling process
                self.process.poll()

    def process_run_over(self):
        # 當子程序結束時呼叫，停止計時器並清理資源
        # Called when subprocess finishes, stop timer and clean up resources
        jeditor_logger.info("ShellManager process_run_over")
        if self.timer is not None:
            self.timer.stop()  # 停止定時器 / stop QTimer
        self.exit_program()  # 結束程序並清理 / terminate process and cleanup
        self.main_window.exec_shell = None  # 重置 main_window 的 exec_shell / reset exec_shell reference
        if self.after_done_function is not None:
            self.after_done_function()  # 執行結束後的回呼函數 / run callback if provided

    # exit program: 將執行旗標設為 False，清理執行緒、佇列與子程序
    # exit program: set running flag to False, clean threads, queues, and subprocess
    def exit_program(self) -> None:
        jeditor_logger.info("ShellManager exit_program")
        self.still_run_shell = False  # 停止讀取迴圈 / stop reading loop
        # 等待讀取執行緒結束 / Wait for reader threads to finish
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread.join(timeout=2)
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread.join(timeout=2)
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()  # 清空輸出佇列 / clear output queues
        if self.process is not None:
            self.process.terminate()  # 終止子程序 / terminate subprocess
            try:
                self.process.wait(timeout=5)
            except subprocess.TimeoutExpired:
                self.process.kill()
            if self.code_result is not None:
                text_cursor = self.code_result.textCursor()
                text_format = QTextCharFormat()
                text_format.setForeground(actually_color_dict.get("normal_output_color"))
                # 顯示退出代碼 / show exit code
                text_cursor.insertText(f"Shell command exit with code {self.process.returncode}", text_format)
                text_cursor.insertBlock()
            self.process = None  # 清空 process 物件 / reset process object

    def print_and_clear_queue(self) -> None:
        # 清空 stdout 與 stderr 佇列
        # Reset stdout and stderr queues
        jeditor_logger.info("ShellManager print_and_clear_queue")
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self) -> None:
        # 從子程序讀取標準輸出並放入佇列
        # Continuously read stdout from subprocess and put into queue
        jeditor_logger.info("ShellManager read_program_output_from_process")
        try:
            while self.still_run_shell:
                program_output_data = self.process.stdout.readline(
                    self.program_buffer) \
                    .decode(self.program_encoding, "replace")  # 解碼輸出 / decode output
                if not program_output_data and self.process.poll() is not None:
                    break
                if self.process:
                    self.process.stdout.flush()  # 清空緩衝區 / flush buffer
                self.run_output_queue.put_nowait(program_output_data)  # 放入輸出佇列 / enqueue stdout
        except (OSError, ValueError):
            pass

    def read_program_error_output_from_process(self) -> None:
        # 從子程序讀取錯誤輸出並放入佇列
        # Continuously read stderr from subprocess and put into queue
        jeditor_logger.info("ShellManager read_program_error_output_from_process")
        try:
            while self.still_run_shell:
                program_error_output_data = self.process.stderr.readline(
                    self.program_buffer) \
                    .decode(self.program_encoding, "replace")  # 解碼錯誤輸出 / decode stderr
                if not program_error_output_data and self.process.poll() is not None:
                    break
                if self.process:
                    self.process.stderr.flush()  # 清空緩衝區 / flush buffer
                self.run_error_queue.put_nowait(program_error_output_data)  # 放入錯誤佇列 / enqueue stderr
        except (OSError, ValueError):
            pass



# 匯入自訂錯誤訊息與例外類別
# Import custom error messages and exception class

import sys
import shutil
from pathlib import Path

# 匯入自訂錯誤訊息與例外類別
# Import custom error messages and exception class
from je_editor.utils.exception.exception_tags import compiler_not_found_error
from je_editor.utils.exception.exceptions import JEditorExecException
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 平台常數 / Platform constants
WINDOWS_PLATFORMS = ("win32", "cygwin", "msys")


def is_windows() -> bool:
    """判斷是否為 Windows 平台 / Check if current platform is Windows"""
    return sys.platform in WINDOWS_PLATFORMS


def get_venv_path(base_path: Path | None = None) -> Path:
    """
    取得虛擬環境的 Python 執行路徑
    Get the Python executable path within a virtual environment

    :param base_path: 專案根目錄，預設為 cwd / Project root, defaults to cwd
    :return: venv 下的 Scripts 或 bin 路徑 / Scripts or bin path under venv
    """
    if base_path is None:
        base_path = Path.cwd()
    if is_windows():
        return base_path / "venv" / "Scripts"
    return base_path / "venv" / "bin"


def check_and_choose_venv(venv_path: Path) -> str:
    """
    功能說明 (Function Description):
    檢查虛擬環境 (venv) 路徑，並嘗試尋找可用的 Python 編譯器。
    Check the virtual environment (venv) path and try to find a usable Python interpreter.

    :param venv_path: 虛擬環境的路徑 / path to the virtual environment
    :return: Python 編譯器的完整路徑 / full path to the Python interpreter
    :raise JEditorExecException: 若找不到 Python 編譯器 / if no Python interpreter is found
    """

    jeditor_logger.info(f"check_venv.py check_and_choose_venv venv_path: {venv_path}")

    # 如果 venv_path 是資料夾且存在
    # If venv_path is a directory and exists
    if venv_path.is_dir() and venv_path.exists():
        # 先嘗試在該路徑下尋找 python3
        # Try to find python3 in the given path
        compiler_path = shutil.which("python3", path=str(venv_path))
        if compiler_path:
            return compiler_path

        # 如果找不到 python3，再找 python
        # If python3 not found, try python
        compiler_path = shutil.which("python", path=str(venv_path))
        if compiler_path:
            return compiler_path

    # 如果 venv_path 無效或都找不到，最後再嘗試系統預設的 python3 或 python
    # If venv_path is invalid or no interpreter found, try system default python3 or python
    compiler_path = shutil.which("python3") or shutil.which("python")

    # 如果還是找不到，拋出例外
    # If still not found, raise exception
    if compiler_path is None:
        raise JEditorExecException(compiler_not_found_error)

    return compiler_path



from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.english import english_word_dict
from je_editor.utils.multi_language.traditional_chinese import traditional_chinese_word_dict


class LanguageWrapper(object):
    """
    功能說明 (Function Description):
    - 提供一個語言包裝器，用來管理目前使用的語言與對應的字典。
    - A language wrapper to manage the current language and its corresponding dictionary.
    """

    def __init__(self):
        # 初始化時記錄日誌
        # Log initialization
        jeditor_logger.info("Init LanguageWrapper")

        # 預設語言為 English
        # Default language is English
        self.language: str = "English"

        # 可選語言字典對照表
        # Mapping of available languages to their word dictionaries
        self.choose_language_dict = {
            "English": english_word_dict,
            "Traditional_Chinese": traditional_chinese_word_dict
        }

        # 根據目前語言選擇對應字典
        # Select the dictionary based on current language
        self.language_word_dict: dict = self.choose_language_dict.get(self.language, english_word_dict)

    def reset_language(self, language) -> None:
        """
        重設語言 (Reset the language)
        :param language: 任何已註冊的語言鍵 / Any registered language key
        """
        jeditor_logger.info(f"LanguageWrapper reset_language language: {language}")

        # 檢查輸入是否為支援的語言（包含插件註冊的語言）
        # Check if the input language is supported (including plugin-registered languages)
        if language in self.choose_language_dict:
            # 更新語言與對應字典
            # Update language and corresponding dictionary
            self.language = language
            self.language_word_dict = self.choose_language_dict.get(self.language, english_word_dict)


# 建立一個全域的 LanguageWrapper 實例
# Create a global instance of LanguageWrapper
language_wrapper = LanguageWrapper()



from PySide6.QtWidgets import QWidget, QLineEdit, QPushButton, QBoxLayout, QHBoxLayout

from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class SearchResultBox(QWidget):
    """
    搜尋結果對話框
    Search result dialog box
    """

    def __init__(self):
        jeditor_logger.info("Init SearchResultBox")
        super().__init__()

        # 垂直佈局 (上到下)
        # Vertical layout (top to bottom)
        self.box_layout = QBoxLayout(QBoxLayout.Direction.TopToBottom)

        # 搜尋輸入框 / Search input field
        self.search_input = QLineEdit()

        # 下一個搜尋按鈕 / Next search button
        self.search_next_button = QPushButton()
        self.search_next_button.setText(language_wrapper.language_word_dict.get("search_next_dialog_pushbutton"))

        # 上一個搜尋按鈕 / Previous search button
        self.search_back_button = QPushButton()
        self.search_back_button.setText(language_wrapper.language_word_dict.get("search_back_dialog_pushbutton"))

        # 水平佈局 (放置前後搜尋按鈕)
        # Horizontal layout (for back/next buttons)
        self.box_h_layout = QHBoxLayout()
        self.box_h_layout.addWidget(self.search_back_button)
        self.box_h_layout.addWidget(self.search_next_button)

        # 將元件加入主佈局
        # Add widgets to main layout
        self.box_layout.addWidget(self.search_input)
        self.box_layout.addLayout(self.box_h_layout)

        # 設定視窗標題
        # Set window title
        self.setWindowTitle("Search Result")

        # 套用主佈局
        # Apply main layout
        self.setLayout(self.box_layout)



from queue import Queue


class AIConfig(object):
    """
    AIConfig 類別：用來管理 AI 模型的設定與訊息佇列
    AIConfig class: manages AI model configuration and message queue
    """

    def __init__(self):
        # 當前 AI 模型的系統提示詞 (system prompt)
        # Current AI model system prompt
        self.current_ai_model_system_prompt: str = ""

        # 可選擇的 AI 模型設定字典
        # Dictionary of choosable AI model configurations
        # 結構: { "AI_model": { "ai_base_url": ..., "ai_api_key": ..., "chat_model": ..., "prompt_template": ... } }
        self.choosable_ai: dict[str, dict[str, str]] = {
            "AI_model": {
                "ai_base_url": "",  # AI 服務的基礎 URL / Base URL of AI service
                "ai_api_key": "",  # API 金鑰 / API key
                "chat_model": "",  # 模型名稱 / Model name
                "prompt_template": "",  # 提示詞模板 / Prompt template
            }
        }

        # 訊息佇列，用來暫存與 AI 的互動訊息
        # Message queue for storing AI interaction messages
        self.message_queue = Queue()


# 建立全域唯一的 AIConfig 實例
# Create a global singleton instance of AIConfig
ai_config = AIConfig()



import sys
import threading
import time
from pathlib import Path

from watchdog.observers import Observer

from je_editor.code_scan.watchdog_implement import RuffPythonFileChangeHandler


class WatchdogThread(threading.Thread):
    """
    A thread that runs a watchdog observer to monitor file changes.
    使用 watchdog 監控檔案變化的執行緒。
    """

    def __init__(self, check_path: str):
        """
        :param check_path: Path to monitor (directory or file)
                           要監控的路徑（資料夾或檔案）
        """
        super().__init__(daemon=True)  # 設為 daemon，主程式結束時自動退出
        self.check_path = Path(check_path).resolve()
        self.ruff_handler = RuffPythonFileChangeHandler()
        self.running = True
        self.observer = Observer()

    def run(self):
        """Start the watchdog observer loop."""
        if not self.check_path.exists():
            print(f"[Error] Path does not exist: {self.check_path}", file=sys.stderr)
            return

        # 設定監控
        self.observer.schedule(self.ruff_handler, str(self.check_path), recursive=True)
        self.observer.start()
        print(f"[Watchdog] Monitoring started on {self.check_path}")

        try:
            while self.running:
                time.sleep(1)
                # 這裡可以加上 queue 輸出處理
                self._process_ruff_output()
        except KeyboardInterrupt:
            print("[Watchdog] Interrupted by user")
        finally:
            self.observer.stop()
            self.observer.join()
            print("[Watchdog] Monitoring stopped")

    def stop(self):
        """Stop the watchdog thread safely."""
        self.running = False

    def _process_ruff_output(self):
        """Process stdout/stderr queues from Ruff threads."""
        while not self.ruff_handler.stdout_queue.empty():
            line = self.ruff_handler.stdout_queue.get()
            print(f"[Ruff STDOUT] {line}")

        while not self.ruff_handler.stderr_queue.empty():
            line = self.ruff_handler.stderr_queue.get()
            print(f"[Ruff STDERR] {line}", file=sys.stderr)


if __name__ == '__main__':
    # 預設監控當前目錄
    path_to_watch = "."
    watchdog_thread = WatchdogThread(path_to_watch)
    watchdog_thread.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        print("[Main] Stopping watchdog...")
        watchdog_thread.stop()
        watchdog_thread.join()



# 匯入未來功能，允許延遲型別註解 (Python 3.7+ 常用)
# Import future feature: postponed evaluation of type annotations
from __future__ import annotations

# 僅用於型別檢查，避免循環匯入
# For type checking only (avoids circular imports)
from typing import TYPE_CHECKING

# 匯入 Qt 動作與訊息框
# Import QAction and QMessageBox from PySide6
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMessageBox

# 匯入使用者設定字典，用來保存語言設定
# Import user settings dictionary for saving language preferences
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
# 匯入日誌紀錄器
# Import logger instance
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 僅在型別檢查時匯入 EditorMain，避免循環依賴
# Import EditorMain only for type checking (avoids circular dependency)
if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

# 匯入多語言包裝器，用於 UI 多語言顯示
# Import multi-language wrapper for UI localization
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


# 設定語言選單
# Set up the Language menu
def set_language_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_language_server.py set_language_menu ui_we_want_to_set: {ui_we_want_to_set}")

    # 建立 Language 選單
    # Create Language menu
    ui_we_want_to_set.language_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("language_menu_label")
    )

    # 建立「切換到英文」動作
    # Add "Switch to English" action
    ui_we_want_to_set.language_menu.change_to_english_language_action = QAction(
        language_wrapper.language_word_dict.get("language_menu_bar_english"),
        ui_we_want_to_set.language_menu
    )
    ui_we_want_to_set.language_menu.change_to_english_language_action.triggered.connect(
        lambda: set_language("English", ui_we_want_to_set)
    )

    # 建立「切換到繁體中文」動作
    # Add "Switch to Traditional Chinese" action
    ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action = QAction(
        language_wrapper.language_word_dict.get("language_menu_bar_traditional_chinese"),
        ui_we_want_to_set.language_menu
    )
    ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action.triggered.connect(
        lambda: set_language("Traditional_Chinese", ui_we_want_to_set)
    )

    # 將動作加入選單
    # Add actions to the menu
    ui_we_want_to_set.language_menu.addAction(
        ui_we_want_to_set.language_menu.change_to_english_language_action
    )
    ui_we_want_to_set.language_menu.addAction(
        ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action
    )

    # 動態加入插件註冊的語言
    # Dynamically add plugin-registered languages
    from je_editor.plugins import get_all_natural_languages
    for lang_key, lang_info in get_all_natural_languages().items():
        display_name = lang_info.get("display_name", lang_key)
        action = QAction(display_name, ui_we_want_to_set.language_menu)
        action.triggered.connect(
            lambda checked=False, lk=lang_key: set_language(lk, ui_we_want_to_set)
        )
        ui_we_want_to_set.language_menu.addAction(action)


# 設定語言
# Set the application language
def set_language(language: str, ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_language_server.py set_language "
                        f"language: {language} "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")

    # 重設語言 (更新多語言字典)
    # Reset language (update multi-language dictionary)
    language_wrapper.reset_language(language)

    # 更新使用者設定，保存語言偏好
    # Update user settings dictionary to persist language preference
    user_setting_dict.update({"language": language})

    # 顯示提示訊息，提醒使用者需要重新啟動程式
    # Show a message box to remind user to restart the application
    message_box = QMessageBox(ui_we_want_to_set)
    message_box.setText(language_wrapper.language_word_dict.get("language_menu_bar_please_restart_messagebox"))
    message_box.show()



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction
from frontengine import FrontEngineMainUI

from je_editor.pyside_ui.code.variable_inspector.inspector_gui import VariableInspector
from je_editor.pyside_ui.main_ui.ai_widget.chat_ui import ChatUI
from je_editor.pyside_ui.main_ui.ipython_widget.ipython_console import IpythonWidget
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


def set_tab_tools_menu(ui_we_want_to_set: EditorMain) -> None:
    ui_we_want_to_set.tab_menu.tools_menu = ui_we_want_to_set.tab_menu.addMenu(
        language_wrapper.language_word_dict.get("tab_menu_tools_submenu_label")
    )

    # === IPython 分頁 ===
    # === IPython Tab ===
    ui_we_want_to_set.tab_menu.tools_menu.add_ipython_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_ipython_tab_name"))
    ui_we_want_to_set.tab_menu.tools_menu.add_ipython_action.triggered.connect(
        lambda: add_ipython_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.tools_menu.addAction(ui_we_want_to_set.tab_menu.tools_menu.add_ipython_action)

    # === Variable Inspector 分頁 ===
    # === Variable Inspector Tab ===
    ui_we_want_to_set.tab_menu.tools_menu.add_variable_inspector_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_variable_inspector_tab_name"))
    ui_we_want_to_set.tab_menu.tools_menu.add_variable_inspector_ui_action.triggered.connect(
        lambda: add_variable_inspector_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.tools_menu.addAction(
        ui_we_want_to_set.tab_menu.tools_menu.add_variable_inspector_ui_action)

    # === Front Engine 分頁 ===
    # === Front Engine Tab ===
    ui_we_want_to_set.tab_menu.tools_menu.add_frontengine_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_frontengine_label"))
    ui_we_want_to_set.tab_menu.tools_menu.add_frontengine_action.triggered.connect(
        lambda: add_frontengine_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.tools_menu.addAction(ui_we_want_to_set.tab_menu.tools_menu.add_frontengine_action)

    # === Chat UI 分頁 ===
    # === Chat UI Tab ===
    ui_we_want_to_set.tab_menu.tools_menu.add_chat_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_chat_ui_tab_name"))
    ui_we_want_to_set.tab_menu.tools_menu.add_chat_ui_action.triggered.connect(
        lambda: add_chat_ui_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.tools_menu.addAction(ui_we_want_to_set.tab_menu.tools_menu.add_chat_ui_action)


def add_ipython_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 IPython 分頁
    # Log: add an IPython tab
    jeditor_logger.info(f"build_tab_menu.py add ipython tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增 IPython 互動式分頁
    # Add an IPython interactive tab
    ui_we_want_to_set.tab_widget.addTab(
        IpythonWidget(ui_we_want_to_set),  # 建立 IPython 元件 / Create IPython widget
        f"{language_wrapper.language_word_dict.get('tab_menu_ipython_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )


def add_variable_inspector_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Variable Inspector 分頁
    # Log: add a Variable Inspector tab
    jeditor_logger.info(f"build_tab_menu.py add variable inspector tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增變數檢查器分頁
    # Add a variable inspector tab
    ui_we_want_to_set.tab_widget.addTab(
        VariableInspector(),  # 建立變數檢查器元件 / Create Variable Inspector widget
        f"{language_wrapper.language_word_dict.get('tab_menu_variable_inspector_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )


def add_frontengine_tab(ui_we_want_to_set: EditorMain):
    # 新增 FrontEngine 分頁
    # Add FrontEngine tab
    jeditor_logger.info(f"build_tab_menu.py add frontengine tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        FrontEngineMainUI(show_system_tray_ray=False, redirect_output=False),
        f"{language_wrapper.language_word_dict.get('tab_menu_frontengine_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")


def add_chat_ui_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Chat UI 分頁
    # Log: add a Chat UI tab
    jeditor_logger.info(f"build_tab_menu.py add chat_ui tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增聊天分頁
    # Add a chat tab into the main editor
    ui_we_want_to_set.tab_widget.addTab(
        ChatUI(ui_we_want_to_set),  # 建立聊天元件 / Create Chat UI widget
        f"{language_wrapper.language_word_dict.get('tab_menu_chat_ui_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )



import ast

from PySide6.QtCore import QAbstractTableModel, Qt, QTimer, QSortFilterProxyModel
from PySide6.QtWidgets import QTableView, QVBoxLayout, QWidget, QLineEdit, QLabel

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class VariableModel(QAbstractTableModel):
    """
    變數模型：負責管理與顯示 Python 全域變數
    Variable model: manages and displays Python global variables
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        self.variables = []  # 儲存變數資訊 [名稱, 型別, 值字串, 真實值]
        # Store variable info [name, type, repr(value), actual value]

    def update_data(self):
        """
        更新變數清單，從全域變數中擷取
        Update variable list from globals()
        """
        parent_widget = self.parent()
        # 避免在 table 正在互動時更新，造成衝突
        # Avoid updating while table is in active state
        if parent_widget and getattr(parent_widget, "table", None):
            if parent_widget.table.state() != QTableView.State.NoState:
                return

        vars_dict = globals()
        self.beginResetModel()
        self.variables = [
            [name, type(value).__name__, repr(value), value]
            for name, value in vars_dict.items()
            if not name.startswith("__")  # 過濾內建變數 / filter out built-in variables
        ]
        self.endResetModel()

    def rowCount(self, parent=None):
        # 回傳變數數量 / return number of variables
        return len(self.variables)

    def columnCount(self, parent=None):
        # 固定三欄：名稱、型別、值 / fixed 3 columns: name, type, value
        return 3

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        # 提供表格顯示的資料
        # Provide data for table display
        if not index.isValid() or not (0 <= index.row() < len(self.variables)):
            return None
        if role == Qt.ItemDataRole.DisplayRole:
            return self.variables[index.row()][index.column()]
        return None

    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
        # 設定表頭文字 (支援多語系)
        # Set header labels (multi-language support)
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return [
                language_wrapper.language_word_dict.get("variable_inspector_var_name"),
                language_wrapper.language_word_dict.get("variable_inspector_var_type"),
                language_wrapper.language_word_dict.get("variable_inspector_var_value")
            ][section]
        return None

    def flags(self, index):
        # 設定欄位屬性，僅允許「值」欄可編輯
        # Set column flags, only "value" column is editable
        if not index.isValid() or not (0 <= index.row() < len(self.variables)):
            return Qt.ItemFlag.NoItemFlags
        base = super().flags(index)
        if index.column() == 2:  # 只允許編輯值欄 / only value column editable
            base |= Qt.ItemFlag.ItemIsEditable
        return base

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        """
        更新變數值，並同步到全域變數
        Update variable value and sync to globals()
        """
        if role == Qt.ItemDataRole.EditRole and index.column() == 2:

            if value == "":
                return False

            old_repr = self.variables[index.row()][2]
            if str(value) == old_repr:
                return False

            var_name = self.variables[index.row()][0]
            try:
                # 嘗試將輸入轉換為 Python 物件
                # Try to evaluate input as Python object
                new_value = ast.literal_eval(value)
            except (ValueError, SyntaxError):
                # 若失敗則當作字串處理
                # If failed, treat as string
                new_value = value

            if new_value == self.variables[index.row()][3]:
                return False

            # 更新全域變數與模型資料
            # Update globals and model data
            globals()[var_name] = new_value
            self.variables[index.row()][2] = repr(new_value)
            self.variables[index.row()][3] = new_value
            self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
            return True
        return False


class VariableProxy(QSortFilterProxyModel):
    """
    過濾代理模型：支援搜尋與編輯轉發
    Proxy model: supports filtering and forwards editing
    """

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        # 將編輯操作轉發到原始模型
        # Forward editing to source model
        return self.sourceModel().setData(
            self.mapToSource(index), value, role
        )


class VariableInspector(QWidget):
    """
    變數檢視器：提供 GUI 介面顯示與搜尋全域變數
    Variable inspector: GUI interface to display and search global variables
    """

    def __init__(self):
        super().__init__()
        self.setWindowTitle(language_wrapper.language_word_dict.get("variable_inspector_title"))
        layout = QVBoxLayout(self)

        # 搜尋框 / search box
        self.search_box = QLineEdit()
        self.search_box.setPlaceholderText(language_wrapper.language_word_dict.get("variable_inspector_search"))
        layout.addWidget(QLabel(language_wrapper.language_word_dict.get("variable_inspector_search")))
        layout.addWidget(self.search_box)

        # 模型與代理模型 / model and proxy model
        self.model = VariableModel(parent=self)
        self.proxy_model = VariableProxy(self)
        self.proxy_model.setSourceModel(self.model)
        self.proxy_model.setFilterCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)  # 不分大小寫 / case-insensitive
        self.proxy_model.setFilterKeyColumn(0)  # 僅針對變數名稱過濾 / filter by variable name

        # 表格顯示 / table view
        self.table = QTableView()
        self.table.setModel(self.proxy_model)
        self.table.horizontalHeader().setStretchLastSection(True)
        self.table.horizontalHeader().setSectionResizeMode(
            self.table.horizontalHeader().ResizeMode.Stretch
        )
        layout.addWidget(self.table, stretch=1)

        # 綁定搜尋框輸入事件 / bind search box input
        self.search_box.textChanged.connect(
            self.proxy_model.setFilterFixedString
        )

        # 定時更新變數清單 (每 500ms)
        # Periodically update variable list (every 500ms)
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.model.update_data)
        self.timer.start(500)



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDockWidget

from je_editor.utils.logging.loggin_instance import jeditor_logger


class DestroyDock(QDockWidget):
    """
    DestroyDock 繼承自 QDockWidget，主要用於建立一個可停駐的視窗，
    並在關閉時自動釋放資源與記錄日誌。

    DestroyDock inherits from QDockWidget, mainly used to create a dockable window,
    and automatically release resources and log events when closed.
    """

    def __init__(self):
        # 初始化時記錄日誌 / Log initialization
        jeditor_logger.info("Init DestroyDock")
        super().__init__()

        # 設定允許停駐的區域 (可停駐在所有邊)
        # Allow docking in all areas (top, bottom, left, right)
        self.setAllowedAreas(Qt.DockWidgetArea.AllDockWidgetAreas)

        # 設定屬性：當視窗關閉時，自動刪除物件以釋放記憶體
        # Set attribute: delete object on close to free memory
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

    def closeEvent(self, event) -> None:
        """
        覆寫 closeEvent，在關閉時額外處理：
        - 記錄關閉事件
        - 關閉內部 widget
        - 呼叫父類別的 close()

        Override closeEvent to:
        - Log the close event
        - Close the internal widget
        - Call parent close()
        """
        # 記錄關閉事件 / Log close event
        jeditor_logger.info(f"DestroyDock closeEvent event: {event}")

        # 關閉 dock 內部的 widget，確保資源釋放
        # Close the internal widget to ensure resource release
        if self.widget() is not None:
            self.widget().close()

        # 呼叫父類別的 closeEvent()，完成 Qt 預設的關閉流程
        # Call parent closeEvent() to complete default Qt closing process
        super().closeEvent(event)



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction

from je_editor.pyside_ui.git_ui.code_diff_compare.code_diff_viewer_widget import DiffViewerWidget

from je_editor.pyside_ui.git_ui.git_client.git_branch_tree_widget import GitTreeViewGUI

from je_editor.pyside_ui.git_ui.git_client.git_client_gui import GitGui

from je_editor.utils.logging.loggin_instance import jeditor_logger

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

def set_tab_git_menu(ui_we_want_to_set: EditorMain) -> None:
    ui_we_want_to_set.tab_menu.git_menu = ui_we_want_to_set.tab_menu.addMenu(
        language_wrapper.language_word_dict.get("tab_menu_git_submenu_label")
    )
    # === Git Client 分頁 ===
    # === Git Client Tab ===
    ui_we_want_to_set.tab_menu.git_menu.add_git_client_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_git_client_tab_name"))
    ui_we_want_to_set.tab_menu.git_menu.add_git_client_ui_action.triggered.connect(
        lambda: add_git_client_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.git_menu.addAction(ui_we_want_to_set.tab_menu.git_menu.add_git_client_ui_action)

    # === Git Branch Tree 分頁 ===
    # === Git Branch Tree Tab ===
    ui_we_want_to_set.tab_menu.git_menu.add_git_branch_view_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_git_branch_tree_view_tab_name"))
    ui_we_want_to_set.tab_menu.git_menu.add_git_branch_view_ui_action.triggered.connect(
        lambda: add_git_tree_view_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.git_menu.addAction(ui_we_want_to_set.tab_menu.git_menu.add_git_branch_view_ui_action)

    # === Code Diff Viewer 分頁 ===
    # === Code Diff Viewer Tab ===
    ui_we_want_to_set.tab_menu.git_menu.add_code_diff_viewer_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_code_diff_viewer_tab_name"))
    ui_we_want_to_set.tab_menu.git_menu.add_code_diff_viewer_ui_action.triggered.connect(
        lambda: add_code_diff_compare_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.git_menu.addAction(ui_we_want_to_set.tab_menu.git_menu.add_code_diff_viewer_ui_action)

def add_git_client_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Git Client 分頁
    # Log: add a Git Client tab
    jeditor_logger.info(f"build_tab_menu.py add git client tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增 Git 客戶端分頁
    # Add a Git client tab
    ui_we_want_to_set.tab_widget.addTab(
        GitGui(),  # 建立 Git GUI 元件 / Create Git GUI widget
        f"{language_wrapper.language_word_dict.get('tab_menu_git_client_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )


def add_git_tree_view_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Git Branch Tree 分頁
    # Log: add a Git Branch Tree tab
    jeditor_logger.info(f"build_tab_menu.py add git tree view tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增 Git 分支樹狀圖分頁
    # Add a Git branch tree view tab
    ui_we_want_to_set.tab_widget.addTab(
        GitTreeViewGUI(),  # 建立 Git Tree View 元件 / Create Git Tree View widget
        f"{language_wrapper.language_word_dict.get('tab_menu_git_branch_tree_view_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )

def add_code_diff_compare_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Code Diff Compare 分頁
    # Log: add a Code Diff Compare tab
    jeditor_logger.info(f"build_tab_menu.py add code diff compare tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增程式碼差異比較分頁
    # Add a code diff comparison tab
    ui_we_want_to_set.tab_widget.addTab(
        DiffViewerWidget(),  # 建立程式碼比對元件 / Create Code Diff Viewer widget
        f"{language_wrapper.language_word_dict.get('tab_code_diff_viewer_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtWidgets import QMenu

from je_editor.pyside_ui.browser.main_browser_widget import MainBrowserWidget
from je_editor.pyside_ui.main_ui.console_widget.console_gui import ConsoleWidget
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.menu.tab_menu.build_tab_git_menu import set_tab_git_menu
from je_editor.pyside_ui.main_ui.menu.tab_menu.build_tab_tools_menu import set_tab_tools_menu
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

from PySide6.QtGui import QAction

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_tab_menu(ui_we_want_to_set: EditorMain) -> None:
    """
    設定主編輯器的分頁選單
    Set up the tab menu for the main editor
    """
    jeditor_logger.info(f"build_tab_menu.py set_tab_menu ui_we_want_to_set:{ui_we_want_to_set}")

    # 建立 Tab 選單 (主容器)
    # Create the Tab menu (main container)
    ui_we_want_to_set.tab_menu: QMenu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("tab_menu_label")
    )

    # === Editor 分頁 ===
    # === Editor Tab ===
    ui_we_want_to_set.tab_menu.add_editor_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_editor_label"))
    ui_we_want_to_set.tab_menu.add_editor_action.triggered.connect(
        lambda: add_editor_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_editor_action)

    # === Web 瀏覽器分頁 ===
    # === Web Browser Tab ===
    ui_we_want_to_set.tab_menu.add_web_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_web_label"))
    ui_we_want_to_set.tab_menu.add_web_action.triggered.connect(
        lambda: add_web_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_web_action)

    # === Console 分頁 ===
    # === Console Tab ===
    ui_we_want_to_set.tab_menu.add_console_widget_ui_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_console_widget_tab_name"))
    ui_we_want_to_set.tab_menu.add_console_widget_ui_action.triggered.connect(
        lambda: add_console_widget_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_console_widget_ui_action)

    set_tab_tools_menu(ui_we_want_to_set=ui_we_want_to_set)
    set_tab_git_menu(ui_we_want_to_set=ui_we_want_to_set)


# === 以下為各分頁新增函式 ===
# === Functions to add each tab ===

def add_editor_tab(ui_we_want_to_set: EditorMain):
    # 新增 Editor 分頁
    # Add Editor tab
    jeditor_logger.info(f"build_tab_menu.py add editor tab ui_we_want_to_set: {ui_we_want_to_set}")
    widget = EditorWidget(ui_we_want_to_set)
    ui_we_want_to_set.tab_widget.addTab(
        widget,
        f"{language_wrapper.language_word_dict.get('tab_menu_editor_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")
    return widget


def add_web_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Web 分頁
    # Log: add a Web tab
    jeditor_logger.info(f"build_tab_menu.py add web tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器的 tab_widget 中新增一個瀏覽器分頁
    # Add a browser tab into the main editor's tab_widget
    ui_we_want_to_set.tab_widget.addTab(
        MainBrowserWidget(),  # 建立瀏覽器元件 / Create browser widget
        f"{language_wrapper.language_word_dict.get('tab_menu_web_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"  # 分頁名稱包含序號 / Tab name with index
    )


def add_console_widget_tab(ui_we_want_to_set: EditorMain):
    # 紀錄日誌：新增 Console 分頁
    # Log: add a Console tab
    jeditor_logger.info(f"build_tab_menu.py add console widget tab ui_we_want_to_set: {ui_we_want_to_set}")
    # 在主編輯器中新增動態控制台分頁
    # Add a dynamic console tab
    ui_we_want_to_set.tab_widget.addTab(
        ConsoleWidget(),  # 建立控制台元件 / Create Console widget
        f"{language_wrapper.language_word_dict.get('tab_menu_console_widget_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}"
    )



# 匯入未來功能，允許延遲型別註解 (Python 3.7+ 常用)
# Import future feature: postponed evaluation of type annotations
from __future__ import annotations

# 僅用於型別檢查，避免循環匯入
# For type checking only (avoids circular imports)
from typing import TYPE_CHECKING

# 匯入編輯器元件
# Import EditorWidget
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
# 匯入顏色設定，用於輸出訊息時的字體顏色
# Import color settings for console/text output
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
# 匯入使用者設定字典，用來保存 Python 解譯器與環境設定
# Import user settings dictionary for saving Python interpreter and environment settings
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
# 匯入日誌紀錄器
# Import logger instance
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 僅在型別檢查時匯入 EditorMain，避免循環依賴
# Import EditorMain only for type checking (avoids circular dependency)
if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

import os
from pathlib import Path

# 匯入 Qt 動作、快捷鍵、文字格式、訊息框、輸入框、檔案對話框
# Import QAction, QKeySequence, QTextCharFormat, QMessageBox, QInputDialog, QFileDialog
from PySide6.QtGui import QAction, QKeySequence, QTextCharFormat
from PySide6.QtWidgets import QMessageBox, QInputDialog, QFileDialog

# 匯入 ShellManager，用於執行系統命令 (建立 venv、pip install 等)
# Import ShellManager for executing shell commands (create venv, pip install, etc.)
from je_editor.pyside_ui.code.shell_process.shell_exec import ShellManager

# 匯入多語言包裝器，用於 UI 多語言顯示
# Import multi-language wrapper for UI localization
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


# 設定 Python 環境選單 (venv menu)
# Set up the Python Environment (venv) menu
def set_venv_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py set_venv_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.venv_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("python_env_menu_label"))

    # 建立虛擬環境 (Ctrl+Shift+V)
    # Create virtual environment
    ui_we_want_to_set.venv_menu.change_language_menu = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_create_venv_label"))
    ui_we_want_to_set.venv_menu.change_language_menu.setShortcut(QKeySequence("Ctrl+Shift+V"))
    ui_we_want_to_set.venv_menu.change_language_menu.triggered.connect(
        lambda: create_venv(ui_we_want_to_set))
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.change_language_menu)

    # pip 升級套件 (Ctrl+Shift+U)
    # pip upgrade package
    ui_we_want_to_set.venv_menu.pip_upgrade_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_pip_upgrade_label"))
    ui_we_want_to_set.venv_menu.pip_upgrade_action.setShortcut(QKeySequence("Ctrl+Shift+U"))
    ui_we_want_to_set.venv_menu.pip_upgrade_action.triggered.connect(
        lambda: pip_install_package_update(ui_we_want_to_set))
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.pip_upgrade_action)

    # pip 安裝套件 (Ctrl+Shift+P)
    # pip install package
    ui_we_want_to_set.venv_menu.pip_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_pip_label"))
    ui_we_want_to_set.venv_menu.pip_action.setShortcut(QKeySequence("Ctrl+Shift+P"))
    ui_we_want_to_set.venv_menu.pip_action.triggered.connect(
        lambda: pip_install_package(ui_we_want_to_set))
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.pip_action)

    # 選擇 Python 解譯器
    # Choose Python interpreter
    ui_we_want_to_set.venv_menu.choose_interpreter_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_choose_interpreter_label"))
    ui_we_want_to_set.venv_menu.choose_interpreter_action.triggered.connect(
        lambda: chose_python_interpreter(ui_we_want_to_set))
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.choose_interpreter_action)


# 建立虛擬環境
# Create virtual environment
def create_venv(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py create_venv ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        venv_path = Path(os.getcwd()) / "venv"
        if not venv_path.exists():
            # 使用 ShellManager 執行 python -m venv venv
            # Use ShellManager to run python -m venv venv
            create_venv_shell = ShellManager(main_window=widget, after_done_function=widget.code_edit.check_env,
                                             shell_encoding=ui_we_want_to_set.encoding)
            create_venv_shell.later_init()
            create_venv_shell.exec_shell([f"{create_venv_shell.compiler_path}", "-m", "venv", "venv"])

            # 在編輯器輸出區顯示訊息
            # Show message in editor output area
            text_cursor = widget.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText(language_wrapper.language_word_dict.get("python_env_menu_creating_venv_message"),
                                   text_format)
            text_cursor.insertBlock()
        else:
            # 如果 venv 已存在，顯示提示訊息
            # If venv exists, show message
            message_box = QMessageBox()
            message_box.setText(language_wrapper.language_word_dict.get("python_env_menu_venv_exists"))
            message_box.exec()


# 使用 pip 安裝套件 (通用函式)
# Run pip install command (general function)
def shell_pip_install(ui_we_want_to_set: EditorMain, pip_install_command_list: list):
    jeditor_logger.info("build_venv_menu.py create_venv "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"pip_install_command_list: {pip_install_command_list}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        venv_path = Path(os.getcwd()) / "venv"
        if not venv_path.exists():
            # 如果 venv 不存在，提醒使用者先建立
            # If venv does not exist, remind user to create it first
            message_box = QMessageBox()
            message_box.setText(language_wrapper.language_word_dict.get("python_env_menu_please_create_venv"))
            message_box.exec()
        else:
            # 彈出輸入框，讓使用者輸入套件名稱
            # Ask user to input package name
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_label")
            )
            if press_ok:
                pip_install_shell = ShellManager(main_window=widget, shell_encoding=ui_we_want_to_set.encoding)
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(pip_install_command_list)


# 偵測 venv 是否存在
# Detect if venv exists
def detect_venv() -> bool:
    jeditor_logger.info("build_venv_menu.py detect_venv")
    venv_path = Path(os.getcwd()) / "venv"
    if not venv_path.exists():
        message_box = QMessageBox()
        message_box.setText(language_wrapper.language_word_dict.get("python_env_menu_please_create_venv"))
        message_box.exec()
        return False
    else:
        return True


# pip 安裝或更新套件
# pip install or update package
def pip_install_package_update(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py pip_install_package_update ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        if detect_venv():
            # 彈出輸入框，讓使用者輸入要安裝或更新的套件名稱
            # Ask user to input package name to install or update
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_or_update_package_messagebox_label")
            )
            if press_ok:
                # 使用 ShellManager 執行 pip install -U
                # Use ShellManager to run pip install -U
                pip_install_shell = ShellManager(
                    main_window=widget,
                    after_done_function=widget.code_edit.check_env,
                    shell_encoding=ui_we_want_to_set.encoding
                )
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(
                    [f"{pip_install_shell.compiler_path}", "-m", "pip", "install", f"{package_text}", "-U"]
                )


# pip 安裝套件 (不加 -U)
# pip install package (without -U)
def pip_install_package(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py pip_install_package ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        if detect_venv():
            # 彈出輸入框，讓使用者輸入要安裝的套件名稱
            # Ask user to input package name
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_label")
            )
            if press_ok:
                # 使用 ShellManager 執行 pip install
                # Use ShellManager to run pip install
                pip_install_shell = ShellManager(
                    main_window=widget,
                    after_done_function=widget.code_edit.check_env,
                    shell_encoding=ui_we_want_to_set.encoding
                )
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(
                    [f"{pip_install_shell.compiler_path}", "-m", "pip", "install", f"{package_text}"]
                )


# 選擇 Python 解譯器
# Choose Python interpreter
def chose_python_interpreter(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_venv_menu.py chose_python_interpreter ui_we_want_to_set: {ui_we_want_to_set}")
    file_path = QFileDialog().getOpenFileName(
        parent=ui_we_want_to_set,
        dir=str(Path.cwd())
    )[0]
    if file_path is not None and file_path != "":
        # 更新主程式的 Python 解譯器路徑
        # Update main editor's Python interpreter path
        ui_we_want_to_set.python_compiler = file_path
        # 同時更新使用者設定，讓下次啟動時能記住
        # Update user settings so it persists across sessions
        user_setting_dict.update({"python_compiler": file_path})



"""
GitHub API 工具 / GitHub API utilities

使用 urllib (stdlib) 存取 GitHub Contents API，列出與下載插件。
Uses urllib (stdlib) to access GitHub Contents API for listing and downloading plugins.
"""
from __future__ import annotations

import json
import urllib.request
import urllib.error
from pathlib import Path

from je_editor.utils.logging.loggin_instance import jeditor_logger

# GitHub API 基底 URL / GitHub API base URL
_GITHUB_API = "https://api.github.com"


def fetch_repo_tree(owner: str, repo: str, branch: str = "main") -> list[dict]:
    """
    遞迴取得 GitHub Repo 的所有檔案清單。
    Recursively fetch all files from a GitHub repo.

    :return: 每個元素為 {"path": "...", "type": "blob"/"tree", "url": "...", "download_url": "..."}
             Each element: {"path": ..., "type": ..., "url": ..., "download_url": ...}
    """
    jeditor_logger.info(f"github_api.py fetch_repo_tree {owner}/{repo} branch={branch}")
    url = f"{_GITHUB_API}/repos/{owner}/{repo}/contents/"
    return _fetch_contents_recursive(url)


def _fetch_contents_recursive(api_url: str) -> list[dict]:
    """
    遞迴取得 GitHub Contents API 的所有項目。
    Recursively fetch all items from GitHub Contents API.
    """
    items = _github_get(api_url)
    if not isinstance(items, list):
        return []

    result = []
    for item in items:
        name = item.get("name", "")
        item_type = item.get("type", "")

        # 跳過隱藏檔案與非插件檔案 / Skip hidden files and non-plugin files
        if name.startswith(".") or name == "LICENSE" or name == "README.md":
            continue

        if item_type == "dir":
            # 遞迴進入目錄 / Recurse into directory
            sub_url = item.get("url", "")
            if sub_url:
                result.extend(_fetch_contents_recursive(sub_url))
        elif item_type == "file" and name.endswith(".py"):
            result.append({
                "name": name,
                "path": item.get("path", ""),
                "download_url": item.get("download_url", ""),
                "size": item.get("size", 0),
                "sha": item.get("sha", ""),
            })

    return result


def fetch_plugin_metadata(download_url: str) -> dict:
    """
    下載插件原始碼並解析 PLUGIN_NAME / PLUGIN_AUTHOR / PLUGIN_VERSION。
    Download plugin source and parse PLUGIN_NAME / PLUGIN_AUTHOR / PLUGIN_VERSION.
    """
    jeditor_logger.info(f"github_api.py fetch_plugin_metadata {download_url}")
    try:
        source = _download_text(download_url)
    except Exception as e:
        jeditor_logger.error(f"Failed to fetch plugin metadata: {e}")
        return {}

    metadata = {}
    for line in source.splitlines():
        line = line.strip()
        for key in ("PLUGIN_NAME", "PLUGIN_AUTHOR", "PLUGIN_VERSION"):
            if line.startswith(f"{key}") and "=" in line:
                # 取 = 右邊的值，移除引號 / Extract value after =, strip quotes
                value = line.split("=", 1)[1].strip().strip("\"'")
                metadata[key] = value
    return metadata


def download_plugin_file(download_url: str, dest_dir: str, file_name: str) -> str:
    """
    下載插件檔案到指定目錄。
    Download a plugin file to the specified directory.

    :return: 儲存的檔案路徑 / Saved file path
    """
    jeditor_logger.info(f"github_api.py download_plugin_file {download_url} -> {dest_dir}/{file_name}")
    dest_path = Path(dest_dir) / file_name
    content = _download_text(download_url)
    dest_path.write_text(content, encoding="utf-8")
    return str(dest_path)


def _github_get(url: str) -> list | dict:
    """
    發送 GET 請求到 GitHub API。
    Send GET request to GitHub API.
    """
    req = urllib.request.Request(url, headers={
        "Accept": "application/vnd.github.v3+json",
        "User-Agent": "JEditor-PluginBrowser",
    })
    with urllib.request.urlopen(req, timeout=15) as resp:
        return json.loads(resp.read().decode("utf-8"))


def _download_text(url: str) -> str:
    """
    下載純文字內容。
    Download plain text content.
    """
    req = urllib.request.Request(url, headers={
        "User-Agent": "JEditor-PluginBrowser",
    })
    with urllib.request.urlopen(req, timeout=30) as resp:
        return resp.read().decode("utf-8")



from __future__ import annotations

from typing import TYPE_CHECKING

# 匯入各種子選單的建構函式
# Import functions to build different sub-menus
from je_editor.pyside_ui.main_ui.menu.dock_menu.build_dock_menu import set_dock_menu
from je_editor.pyside_ui.main_ui.menu.language_menu.build_language_server import set_language_menu
from je_editor.pyside_ui.main_ui.menu.plugin_menu.build_plugin_menu import set_plugin_menu
from je_editor.pyside_ui.main_ui.menu.style_menu.build_style_menu import set_style_menu
from je_editor.pyside_ui.main_ui.menu.tab_menu.build_tab_menu import set_tab_menu
from je_editor.pyside_ui.main_ui.menu.text_menu.build_text_menu import set_text_menu
from je_editor.utils.logging.loggin_instance import jeditor_logger

# 啟用未來註解功能，允許型別提示使用字串前向參照
# Enable future annotations, allowing forward references in type hints
# TYPE_CHECKING 用於避免在執行時載入不必要的模組
# TYPE_CHECKING prevents unnecessary imports at runtime

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
    # 僅在型別檢查時匯入 EditorMain，避免循環依賴
    # Import EditorMain only for type checking

from PySide6.QtWidgets import QMenuBar
# 匯入 QMenuBar，用於建立選單列
# Import QMenuBar to create the menu bar

# 其他子選單建構函式
# Other sub-menu builders
from je_editor.pyside_ui.main_ui.menu.check_style_menu.build_check_style_menu import set_check_menu
from je_editor.pyside_ui.main_ui.menu.file_menu.build_file_menu import set_file_menu
from je_editor.pyside_ui.main_ui.menu.help_menu.build_help_menu import set_help_menu
from je_editor.pyside_ui.main_ui.menu.run_menu.build_run_menu import set_run_menu
from je_editor.pyside_ui.main_ui.menu.python_env_menu.build_venv_menu import set_venv_menu


def set_menu_bar(ui_we_want_to_set: EditorMain) -> None:
    """
    建立主選單列，並依序加入各種子選單
    Build the main menu bar and add all sub-menus in order
    """
    jeditor_logger.info(f"set_menu_bar.py set_menu_bar ui_we_want_to_set: {ui_we_want_to_set}")

    # 建立一個新的選單列
    # Create a new menu bar
    ui_we_want_to_set.menu = QMenuBar()

    # === 加入各種子選單 ===
    # === Add different sub-menus ===
    set_file_menu(ui_we_want_to_set)  # 檔案選單 / File menu
    set_run_menu(ui_we_want_to_set)  # 執行選單 / Run menu
    set_text_menu(ui_we_want_to_set)  # 文字設定選單 / Text menu
    set_check_menu(ui_we_want_to_set)  # 程式碼檢查選單 / Code style check menu
    set_help_menu(ui_we_want_to_set)  # 幫助選單 / Help menu
    set_venv_menu(ui_we_want_to_set)  # Python 虛擬環境選單 / Python venv menu
    set_tab_menu(ui_we_want_to_set)  # 分頁選單 / Tab menu
    set_dock_menu(ui_we_want_to_set)  # Dock 視窗選單 / Dock menu
    set_style_menu(ui_we_want_to_set)  # 介面樣式選單 / Style menu
    set_language_menu(ui_we_want_to_set)  # 語言選單 / Language menu

    # 僅在非擴充模式時建立插件選單（擴充模式如 PyBreeze 會自行建立）
    # Only build plugin menu in standalone mode (extend mode like PyBreeze builds its own)
    if not getattr(ui_we_want_to_set, "extend", False):
        set_plugin_menu(ui_we_want_to_set)

    # 將選單列設定到主視窗
    # Attach the menu bar to the main editor window
    ui_we_want_to_set.setMenuBar(ui_we_want_to_set.menu)



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    # 僅在型別檢查時匯入，避免循環匯入問題
    # Import only for type checking, prevents circular imports
    from je_editor.pyside_ui.main_ui.main_editor import EditorWidget

from je_editor.pyside_ui.code.auto_save.auto_save_thread import CodeEditSaveThread

# --- 全域管理字典 / Global manager dictionaries ---

# 管理每個檔案對應的自動儲存執行緒
# Manage auto-save threads for each file
auto_save_manager_dict: dict = dict()

# 管理檔案是否已經開啟
# Track whether a file is currently open
file_is_open_manager_dict: dict = dict()


def init_new_auto_save_thread(file_path: str, widget: EditorWidget):
    """
    初始化新的自動儲存執行緒
    Initialize a new auto-save thread for the given file and editor widget.

    :param file_path: 檔案路徑 / Path of the file to be auto-saved
    :param widget: 編輯器元件 / Editor widget instance
    """
    # 記錄初始化訊息 (方便除錯)
    # Log initialization info (for debugging)
    jeditor_logger.info(f"auto_save_manager.py init_new_auto_save_thread "
                        f"file_path: {file_path} "
                        f"widget: {widget}")

    # 將目前編輯器綁定的檔案設為 file_path
    # Set the current file of the editor widget
    widget.current_file = file_path

    # 如果該檔案尚未有自動儲存執行緒
    # If no auto-save thread exists for this file
    if auto_save_manager_dict.get(file_path, None) is None:
        # 建立新的自動儲存執行緒，綁定到編輯器
        # Create a new auto-save thread for this file
        widget.code_save_thread = CodeEditSaveThread(
            file_to_save=widget.current_file, editor=widget.code_edit
        )

        # 更新管理字典，記錄該檔案對應的執行緒
        # Update manager dict with the new thread
        auto_save_manager_dict.update({
            file_path: widget.code_save_thread
        })

        # 啟動自動儲存執行緒
        # Start the auto-save thread
        widget.code_save_thread.start()



import logging
from logging.handlers import RotatingFileHandler

# 設定 root logger 的最低層級為 DEBUG
# Set the root logger level to DEBUG
logging.root.setLevel(logging.DEBUG)

# 建立一個名為 "JEditor" 的 logger
# Create a logger named "JEditor"
jeditor_logger = logging.getLogger("JEditor")

# 設定 JEditor logger 的層級為 WARNING (只會輸出 WARNING 以上的訊息)
# Set the JEditor logger level to WARNING (only WARNING and above will be logged)
jeditor_logger.setLevel(logging.WARNING)

# 定義日誌格式：時間 | logger 名稱 | 等級 | 訊息
# Define log format: time | logger name | level | message
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')


class JEditorLoggingHandler(RotatingFileHandler):
    """
    自訂的 Logging Handler，繼承自 RotatingFileHandler
    Custom Logging Handler, inherits from RotatingFileHandler

    功能：
    - 將日誌輸出到檔案 (支援檔案大小輪替)
    - 預設檔名為 JEditor.log
    """

    # redirect logging stderr output to queue (註解說明，但目前未實作)
    # 註解提到要將 stderr 輸出導向 queue，但目前程式碼僅繼承 RotatingFileHandler

    def __init__(self, filename: str = "JEditor.log", mode="w",
                 max_bytes: int = 1073741824, backup_count: int = 0):
        """
        :param filename: 日誌檔案名稱 / log file name
        :param mode: 檔案開啟模式 (預設 w 覆寫) / file open mode (default "w" overwrite)
        :param max_bytes: 單一檔案最大大小 (預設 1GB) / max file size (default 1GB)
        :param backup_count: 保留的備份檔案數量 / number of backup files to keep
        """
        super().__init__(filename=filename, mode=mode, maxBytes=max_bytes, backupCount=backup_count)
        self.formatter = formatter  # 設定日誌格式 / set log formatter
        self.setLevel(logging.DEBUG)  # 設定 handler 層級為 DEBUG / set handler level to DEBUG

    def emit(self, record: logging.LogRecord) -> None:
        """
        實際輸出日誌的方法，這裡直接呼叫父類別的 emit
        Method to emit log records, here just call parent emit
        """
        super().emit(record)


# 建立檔案處理器並加入到 JEditor logger
# Create file handler and add it to JEditor logger
file_handler = JEditorLoggingHandler()
jeditor_logger.addHandler(file_handler)



# 匯入未來功能，允許延遲型別註解 (Python 3.7+ 常用)
# Import future feature: postponed evaluation of type annotations
from __future__ import annotations

# 僅用於型別檢查，避免循環匯入
# For type checking only (avoids circular imports)
from typing import TYPE_CHECKING

# 匯入 Qt 動作與訊息框
# Import QAction and QMessageBox from PySide6
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMessageBox

# 匯入日誌紀錄器
# Import logger instance
from je_editor.utils.logging.loggin_instance import jeditor_logger
# 匯入多語言包裝器
# Import multi-language wrapper for UI localization
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

# 僅在型別檢查時匯入 EditorMain，避免循環依賴
# Import EditorMain only for type checking (avoids circular dependency)
if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


def set_plugin_menu(ui_we_want_to_set: EditorMain) -> None:
    """
    建立插件選單，顯示所有已載入插件的名稱、版本、作者。
    Build Plugin menu showing all loaded plugins with name, version, author.
    同一語言若支援多種副檔名，以子選單呈現。
    If a language plugin supports multiple suffixes, show them in a submenu.
    """
    jeditor_logger.info(f"build_plugin_menu.py set_plugin_menu ui_we_want_to_set: {ui_we_want_to_set}")

    from je_editor.plugins import get_all_plugin_metadata

    # 建立 Plugin 選單
    # Create Plugin menu
    ui_we_want_to_set.plugin_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("plugin_menu_label", "Plugins")
    )

    # 插件瀏覽器入口 / Plugin browser entry
    browse_action = QAction(
        language_wrapper.language_word_dict.get("plugin_browser_tab_name", "Plugin Browser"),
        ui_we_want_to_set.plugin_menu,
    )
    browse_action.triggered.connect(lambda: _open_plugin_browser(ui_we_want_to_set))
    ui_we_want_to_set.plugin_menu.addAction(browse_action)
    ui_we_want_to_set.plugin_menu.addSeparator()

    metadata_list = get_all_plugin_metadata()

    for meta in metadata_list:
        plugin_name = meta.get("name", "Unknown")
        plugin_author = meta.get("author", "")
        plugin_version = meta.get("version", "")
        run_config = meta.get("run_config")

        if run_config is not None:
            suffixes = run_config.get("suffixes", ())
            config_name = run_config.get("name", plugin_name)

            if len(suffixes) > 1:
                # 多種副檔名：建立子選單
                # Multiple suffixes: create a submenu
                sub_menu = ui_we_want_to_set.plugin_menu.addMenu(config_name)

                # 「關於」動作
                # "About" action
                about_action = QAction(
                    language_wrapper.language_word_dict.get("plugin_menu_about", "About"),
                    sub_menu,
                )
                about_action.triggered.connect(
                    _make_about_callback(plugin_name, plugin_version, plugin_author)
                )
                sub_menu.addAction(about_action)
                sub_menu.addSeparator()

                # 每個副檔名一個執行動作
                # One run action per suffix
                for suffix in suffixes:
                    run_action = QAction(
                        language_wrapper.language_word_dict.get(
                            "plugin_menu_run_with", "Run with {name}"
                        ).format(name=f"{config_name} ({suffix})"),
                        sub_menu,
                    )
                    run_action.triggered.connect(
                        _make_run_callback(ui_we_want_to_set, run_config, suffix)
                    )
                    sub_menu.addAction(run_action)
            else:
                # 單一副檔名：直接加入選單
                # Single suffix: add directly to menu
                sub_menu = ui_we_want_to_set.plugin_menu.addMenu(config_name)

                about_action = QAction(
                    language_wrapper.language_word_dict.get("plugin_menu_about", "About"),
                    sub_menu,
                )
                about_action.triggered.connect(
                    _make_about_callback(plugin_name, plugin_version, plugin_author)
                )
                sub_menu.addAction(about_action)
                sub_menu.addSeparator()

                suffix = suffixes[0] if suffixes else ""
                run_action = QAction(
                    language_wrapper.language_word_dict.get(
                        "plugin_menu_run_with", "Run with {name}"
                    ).format(name=config_name),
                    sub_menu,
                )
                run_action.triggered.connect(
                    _make_run_callback(ui_we_want_to_set, run_config, suffix)
                )
                sub_menu.addAction(run_action)
        else:
            # 沒有執行設定的插件（如翻譯插件），只顯示關於
            # Plugins without run config (e.g. translation), show about only
            about_action = QAction(plugin_name, ui_we_want_to_set.plugin_menu)
            about_action.triggered.connect(
                _make_about_callback(plugin_name, plugin_version, plugin_author)
            )
            ui_we_want_to_set.plugin_menu.addAction(about_action)


def _open_plugin_browser(ui_we_want_to_set: EditorMain) -> None:
    """
    開啟插件瀏覽器分頁。
    Open plugin browser tab.
    """
    from je_editor.pyside_ui.main_ui.plugin_browser.plugin_browser_widget import PluginBrowserWidget

    tab_name = language_wrapper.language_word_dict.get("plugin_browser_tab_name", "Plugin Browser")
    ui_we_want_to_set.tab_widget.addTab(
        PluginBrowserWidget(),
        f"{tab_name} {ui_we_want_to_set.tab_widget.count()}"
    )


def _make_about_callback(name: str, version: str, author: str):
    """
    建立顯示插件資訊的回呼函式。
    Create a callback to show plugin info dialog.
    """
    def callback():
        message_box = QMessageBox()
        message_box.setWindowTitle(name)
        message_box.setText(
            f"{name}\n"
            f"Version: {version}\n"
            f"Author: {author}"
        )
        message_box.exec()
    return callback


def _make_run_callback(ui_we_want_to_set: EditorMain, run_config: dict, suffix: str):
    """
    建立使用插件執行設定來執行程式的回呼函式。
    Create a callback to run a program using plugin run config.
    """
    def callback():
        from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
        from je_editor.pyside_ui.code.code_process.code_exec import ExecManager
        from je_editor.pyside_ui.dialog.file_dialog.save_file_dialog import choose_file_get_save_file_path
        from je_editor.pyside_ui.main_ui.menu.run_menu.under_run_menu.utils import \
            please_close_current_running_messagebox

        widget = ui_we_want_to_set.tab_widget.currentWidget()
        if not isinstance(widget, EditorWidget):
            return

        if widget.exec_program is not None:
            please_close_current_running_messagebox(ui_we_want_to_set)
            return

        if not choose_file_get_save_file_path(ui_we_want_to_set):
            return

        code_exec = ExecManager(widget, program_encoding=ui_we_want_to_set.encoding)
        code_exec.later_init()
        code_exec.exec_with_plugin_config(widget.current_file, run_config)
        widget.exec_program = code_exec

    return callback



from pathlib import Path

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QGridLayout, QScrollArea
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.file.save.save_file import write_file
from je_editor.utils.logging.loggin_instance import jeditor_logger


class FullEditorWidget(QWidget):
    """
    FullEditorWidget 提供一個完整的單檔編輯器介面，
    包含程式碼編輯區、捲動支援，以及關閉時自動儲存功能。

    FullEditorWidget provides a full single-file editor interface,
    including code editing area, scroll support, and auto-save on close.
    """

    def __init__(self, current_file: str):
        # 初始化時記錄日誌 / Log initialization
        jeditor_logger.info(f"Init FullEditorWidget current_file: {current_file}")
        super().__init__()

        # ---------------- Init variable 初始化變數 ----------------
        self.current_file = current_file  # 目前編輯的檔案路徑 / Current editing file path

        # ---------------- Attributes 屬性設定 ----------------
        # 設定關閉時自動刪除物件，釋放記憶體
        # Delete object on close to free memory
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

        # ---------------- UI 初始化 ----------------
        self.grid_layout = QGridLayout(self)
        self.setWindowTitle(language_wrapper.language_word_dict.get("application_name"))

        # 建立程式碼編輯器並放入捲動區域
        # Create code editor and put inside scroll area
        self.code_edit = CodeEditor(self)
        self.code_edit_scroll_area = QScrollArea()
        self.code_edit_scroll_area.setWidgetResizable(True)
        self.code_edit_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.code_edit_scroll_area.setWidget(self.code_edit)

        # 將編輯器加入版面配置
        # Add editor to layout
        self.grid_layout.addWidget(self.code_edit_scroll_area, 0, 0)

        # 設定字體樣式 (從使用者設定檔讀取)
        # Set font style (from user settings)
        self.code_edit.setStyleSheet(
            f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
            f"font-family: {user_setting_dict.get('font', 'Lato')};"
        )

    def closeEvent(self, event) -> None:
        """
        覆寫 closeEvent，在關閉視窗時自動儲存檔案內容。
        Override closeEvent to auto-save file content when closing window.
        """
        jeditor_logger.info(f"FullEditorWidget closeEvent event: {event}")
        path = Path(self.current_file)
        if path.exists() and path.is_file():
            # 將編輯器內容寫回檔案 / Write editor content back to file
            write_file(self.current_file, self.code_edit.toPlainText())
        # 呼叫父類別的 closeEvent，完成 Qt 預設流程
        # Call parent closeEvent to complete Qt default process
        super().closeEvent(event)



import logging
from pathlib import Path

from PySide6.QtCore import QTimer, QFileSystemWatcher, Qt
from PySide6.QtGui import QAction
from PySide6.QtWidgets import (
    QFileDialog, QToolBar, QMessageBox, QStatusBar,
    QSplitter, QWidget, QVBoxLayout
)

from je_editor.git_client.commit_graph import CommitGraph
from je_editor.git_client.git_cli import GitCLI
from je_editor.pyside_ui.git_ui.git_client.commit_table import CommitTable
from je_editor.pyside_ui.git_ui.git_client.graph_view import CommitGraphView
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


class GitTreeViewGUI(QWidget):
    """
    GitTreeViewGUI: Git 提交歷史圖形化檢視器
    GitTreeViewGUI: A graphical viewer for Git commit history
    """

    def __init__(self):
        super().__init__()
        self.language_wrapper_get = language_wrapper.language_word_dict.get
        self.setWindowTitle(self.language_wrapper_get("git_graph_title"))

        # === 主版面配置 / Main layout ===
        git_treeview_main_layout = QVBoxLayout(self)
        git_treeview_main_layout.setContentsMargins(0, 0, 0, 0)
        git_treeview_main_layout.setSpacing(0)

        # === 工具列 / Toolbar ===
        git_treeview_toolbar = QToolBar()

        # 開啟 Git 專案動作 / Action to open Git repo
        open_repo_action = QAction(self.language_wrapper_get("git_graph_toolbar_open"), self)
        open_repo_action.triggered.connect(self.open_repo)
        git_treeview_toolbar.addAction(open_repo_action)

        # 刷新圖表動作 / Action to refresh graph
        act_refresh = QAction(self.language_wrapper_get("git_graph_toolbar_refresh"), self)
        act_refresh.triggered.connect(self.refresh_graph)
        git_treeview_toolbar.addAction(act_refresh)

        git_treeview_main_layout.addWidget(git_treeview_toolbar)

        # === 分割器 (左：圖形檢視 / 右：提交表格) ===
        splitter = QSplitter(Qt.Orientation.Horizontal)
        self.graph_view = CommitGraphView()  # 提交圖形檢視器 / commit graph view
        self.commit_table = CommitTable()  # 提交表格 / commit table
        splitter.addWidget(self.graph_view)
        splitter.addWidget(self.commit_table)
        splitter.setSizes([600, 400])  # 左右比例 / left-right ratio
        git_treeview_main_layout.addWidget(splitter, 1)  # stretch=1 表示可伸縮

        # === 狀態列 / Status bar ===
        self.status = QStatusBar()
        git_treeview_main_layout.addWidget(self.status)

        # === Git 相關屬性 ===
        self.repo_path = None
        self.git = None

        # === 檔案監控器 / File system watcher ===
        self.watcher = QFileSystemWatcher()
        self.watcher.directoryChanged.connect(self._on_git_changed)
        self.watcher.fileChanged.connect(self._on_git_changed)

        # === 定時器 (延遲刷新) / Timer for delayed refresh ===
        self.refresh_timer = QTimer()
        self.refresh_timer.setSingleShot(True)
        self.refresh_timer.timeout.connect(self.refresh_graph)

        # === 表格選擇事件 / Table selection event ===
        self.commit_table.selectionModel().selectionChanged.connect(self._on_table_selection)

    def _on_table_selection(self, selected, _):
        """
        當使用者在提交表格中選擇某一列時，讓圖形檢視器聚焦到該行
        When user selects a row in commit table, focus graph view on that row
        """
        if not selected.indexes():
            return
        row = selected.indexes()[0].row()
        self.graph_view.focus_row(row)

    def open_repo(self):
        """
        開啟 Git 專案並初始化圖形檢視
        Open a Git repository and set up the graph view
        """
        path = QFileDialog.getExistingDirectory(self, self.language_wrapper_get("git_graph_menu_open_repo"))
        if not path:
            return
        repo_path = Path(path)
        git = GitCLI(repo_path)
        if not git.is_git_repo():
            # 若不是 Git 專案，顯示警告
            # Show warning if not a Git repo
            QMessageBox.warning(self, self.language_wrapper_get("git_graph_title"),
                                self.language_wrapper_get("git_graph_error_not_git"))
            return
        self.repo_path = repo_path
        self.git = git
        self._setup_watcher()
        self.refresh_graph()

    def _setup_watcher(self):
        """
        設定檔案監控，監控 .git 目錄與相關檔案
        Setup file watcher to monitor .git directory and related files
        """
        self.watcher.removePaths(self.watcher.files())
        self.watcher.removePaths(self.watcher.directories())
        if not self.repo_path:
            return
        git_dir = self.repo_path / ".git"
        if git_dir.exists():
            self.watcher.addPath(str(git_dir))
            for f in ["HEAD", "packed-refs"]:
                fp = git_dir / f
                if fp.exists():
                    self.watcher.addPath(str(fp))
            refs_dir = git_dir / "refs"
            if refs_dir.exists():
                self.watcher.addPath(str(refs_dir))

    def _on_git_changed(self):
        """
        當 Git 目錄變更時，啟動延遲刷新
        When Git directory changes, start delayed refresh
        """
        self.refresh_timer.start(500)

    def refresh_graph(self):
        """
        刷新 Git 提交圖與提交表格
        Refresh Git commit graph and commit table
        """
        if not self.git:
            return
        try:
            refs = self.git.get_all_refs()  # 取得所有 refs
            commits = self.git.get_commits(max_count=500)  # 取得最近 500 筆提交
            graph = CommitGraph()
            graph.build(commits, refs)  # 建立提交圖
            self.graph_view.set_graph(graph)  # 更新圖形檢視
            self.commit_table.set_commits(commits)  # 更新提交表格
        except Exception as e:
            QMessageBox.critical(self, self.language_wrapper_get("git_graph_title"),
                                 f"{self.language_wrapper_get('git_graph_error_exec_failed')}\n{e}")



from __future__ import annotations  # 支援延遲型別註解 (Python 3.7+)

import queue  # 佇列，用於執行緒間傳遞訊息
import subprocess  # 建立與管理子程序 (執行外部程式)
import sys  # 系統相關資訊 (平台、參數等)
from pathlib import Path  # 處理檔案與路徑
from threading import Thread  # 執行緒，用於非同步處理
from typing import Union  # 型別提示：允許多種型別

from PySide6.QtCore import QTimer  # Qt 計時器，用於定時觸發事件
from PySide6.QtGui import QTextCharFormat  # 設定文字格式 (顏色、字型等)
from PySide6.QtWidgets import QTextEdit  # 文字編輯器元件

# 專案內部模組
from je_editor.pyside_ui.code.running_process_manager import run_instance_manager
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.utils.exception.exception_tags import je_editor_init_error
from je_editor.utils.exception.exceptions import JEditorException
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.venv_check.check_venv import check_and_choose_venv, get_venv_path


# Support postponed evaluation of type annotations


class ExecManager(object):
    """
    程式執行管理器
    Execution manager for running code inside the editor.
    負責：
    - 建立子程序 (subprocess)
    - 讀取標準輸出與錯誤輸出
    - 將結果顯示在 QTextEdit
    - 管理計時器與執行緒
    """

    def __init__(
            self,
            main_window: Union[EditorWidget, None] = None,  # 主視窗 (可為 None)
            program_language: str = "python",  # 預設程式語言
            program_encoding: str = "utf-8",  # 預設編碼
            program_buffer: int = 1024,  # 緩衝區大小
    ):
        """
        初始化執行管理器
        Initialize ExecManager
        """
        jeditor_logger.info(f"Init ExecManager "
                            f"main_window: {main_window} "
                            f"program_language: {program_language} "
                            f"program_encoding: {program_encoding} "
                            f"program_buffer: {program_buffer}")
        # 初始化屬性
        self.read_program_error_output_from_thread = None  # 錯誤輸出讀取執行緒
        self.read_program_output_from_thread = None  # 標準輸出讀取執行緒
        self.main_window: EditorWidget = main_window  # 主視窗
        self.compiler_path = None  # 編譯器/直譯器路徑
        self.code_result: Union[QTextEdit, None] = None  # 顯示程式輸出的文字框
        self.code_result_cursor: Union[QTextEdit.textCursor, None] = None
        self.timer: Union[QTimer, None] = None  # 定時器
        self.still_run_program = True  # 程式是否仍在執行
        self.process: Union[subprocess.Popen, None] = None  # 子程序
        self.run_output_queue = queue.Queue()  # 標準輸出佇列
        self.run_error_queue = queue.Queue()  # 錯誤輸出佇列
        self.program_language = program_language
        self.program_encoding = program_encoding
        self.program_buffer = program_buffer
        self.renew_path()  # 設定 Python 直譯器路徑
        run_instance_manager.instance_list.append(self)  # 註冊到全域執行管理器

    def renew_path(self) -> None:
        """更新 Python 直譯器路徑 / Renew compiler path"""
        jeditor_logger.info("ExecManager renew_path")
        if self.main_window is None or self.main_window.python_compiler is None:
            # 如果主視窗沒有指定 Python，則使用虛擬環境
            self.compiler_path = check_and_choose_venv(get_venv_path())
        else:
            self.compiler_path = self.main_window.python_compiler

    def later_init(self) -> None:
        """延遲初始化，設定輸出區與計時器 / Setup code result area and timer"""
        jeditor_logger.info("ExecManager later_init")
        if self.main_window is not None:
            self.code_result: QTextEdit = self.main_window.code_result
            self.timer = QTimer(self.main_window)
        else:
            raise JEditorException(je_editor_init_error)

    def exec_code(self, exec_file_name, exec_prefix: Union[str, list] = None) -> None:
        """
        執行指定檔案
        Execute given file
        :param exec_file_name: 要執行的檔案名稱
        :param exec_prefix: 使用者自定義前綴 (例如 python -m)
        """
        jeditor_logger.info(f"ExecManager exec_code "
                            f"exec_file_name: {exec_file_name} "
                            f"exec_prefix: {exec_prefix}")
        try:
            self.exit_program()  # 確保先結束舊的程式
            self.code_result.setPlainText("")  # 清空輸出區
            file_path = Path(exec_file_name)
            reformat_os_file_path = str(file_path.absolute())
            exec_file = reformat_os_file_path

            # 建立執行參數
            if exec_prefix is None:
                execute_program_param = [self.compiler_path, exec_file]
            else:
                if isinstance(exec_prefix, str):
                    execute_program_param = [self.compiler_path, exec_prefix, exec_file]
                else:
                    execute_program_param = [self.compiler_path] + exec_prefix + [exec_file]

            # 建立子程序 (always use list args with shell=False)
            self.process = subprocess.Popen(
                execute_program_param,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=False
            )
            self.still_run_program = True

            # 啟動輸出讀取執行緒
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            )
            self.read_program_output_from_thread.start()

            # 啟動錯誤讀取執行緒
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            )
            self.read_program_error_output_from_thread.start()

            # 顯示執行的檔案路徑
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText(self.compiler_path + " " + reformat_os_file_path, text_format)
            text_cursor.insertBlock()

            # 啟動定時器，每 10ms 更新輸出
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(10)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()

        except Exception as error:
            # 發生錯誤時顯示錯誤訊息
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText(str(error), text_format)
            text_cursor.insertBlock()
            if self.process is not None:
                self.process.terminate()

    def exec_with_plugin_config(self, exec_file_name: str, run_config: dict) -> None:
        """
        使用插件的 PLUGIN_RUN_CONFIG 執行程式。
        Execute a program using plugin's PLUGIN_RUN_CONFIG.

        支援兩種模式 / Supports two modes:
        1. compile_then_run: 先編譯再執行（如 C/C++/Rust）
           Compile first, then run the output binary
        2. 直接執行（如 Go, Java）
           Direct execution (e.g. go run, java)

        :param exec_file_name: 要執行的檔案名稱 / File to execute
        :param run_config: 插件執行設定字典 / Plugin run config dict
        """
        jeditor_logger.info(f"ExecManager exec_with_plugin_config "
                            f"exec_file_name: {exec_file_name} "
                            f"run_config: {run_config}")
        try:
            self.exit_program()
            self.code_result.setPlainText("")
            file_path = Path(exec_file_name)
            reformat_os_file_path = str(file_path.absolute())

            compiler = run_config.get("compiler", "")
            args = list(run_config.get("args", ()))
            compile_then_run = run_config.get("compile_then_run", False)
            output_flag = run_config.get("output_flag", "")

            if compile_then_run:
                # 先編譯再執行 / Compile then run
                output_path = str(file_path.with_suffix(".exe" if sys.platform == "win32" else ""))

                # 編譯指令 / Compile command
                compile_cmd = [compiler] + args + [reformat_os_file_path]
                if output_flag:
                    compile_cmd += [output_flag, output_path]

                # 顯示編譯指令 / Show compile command
                text_cursor = self.code_result.textCursor()
                text_format = QTextCharFormat()
                text_format.setForeground(actually_color_dict.get("normal_output_color"))
                text_cursor.insertText("[Compile] " + " ".join(compile_cmd), text_format)
                text_cursor.insertBlock()

                # 執行編譯 / Run compilation
                compile_process = subprocess.Popen(
                    compile_cmd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    shell=False,
                )
                stdout, stderr = compile_process.communicate(timeout=60)

                if stderr:
                    text_format_err = QTextCharFormat()
                    text_format_err.setForeground(actually_color_dict.get("error_output_color"))
                    text_cursor.insertText(stderr.decode(self.program_encoding, "replace"), text_format_err)
                    text_cursor.insertBlock()

                if compile_process.returncode != 0:
                    text_cursor.insertText(
                        f"Compilation failed with code {compile_process.returncode}", text_format
                    )
                    text_cursor.insertBlock()
                    return

                if stdout:
                    text_cursor.insertText(stdout.decode(self.program_encoding, "replace"), text_format)
                    text_cursor.insertBlock()

                # 編譯成功，執行輸出檔 / Compilation succeeded, run output
                execute_program_param = [output_path]
                display_cmd = output_path
            else:
                # 直接執行 / Direct execution
                execute_program_param = [compiler] + args + [reformat_os_file_path]
                display_cmd = " ".join(execute_program_param)

            # 建立子程序 / Create subprocess
            self.process = subprocess.Popen(
                execute_program_param,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=False,
            )
            self.still_run_program = True

            # 啟動輸出讀取執行緒 / Start output reader threads
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True,
            )
            self.read_program_output_from_thread.start()

            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True,
            )
            self.read_program_error_output_from_thread.start()

            # 顯示執行指令 / Show run command
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText("[Run] " + display_cmd, text_format)
            text_cursor.insertBlock()

            # 啟動定時器 / Start timer
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(10)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()

        except Exception as error:
            text_cursor = self.code_result.textCursor()
            text_format = QTextCharFormat()
            text_format.setForeground(actually_color_dict.get("normal_output_color"))
            text_cursor.insertText(str(error), text_format)
            text_cursor.insertBlock()
            if self.process is not None:
                self.process.terminate()

    def full_exit_program(self):
        """完全結束程式 / Fully exit program"""
        jeditor_logger.info("ExecManager full_exit_program")
        if self.timer is not None:
            self.timer.stop()
        self.exit_program()
        self.main_window.exec_program = None

    def pull_text(self) -> None:
        jeditor_logger.info("ExecManager pull_text")
        # 從佇列中取出訊息並顯示到 QTextEdit
        # Pull text from queue and put in code result area
        try:
            # 處理標準輸出
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    text_cursor = self.code_result.textCursor()
                    text_format = QTextCharFormat()
                    text_format.setForeground(actually_color_dict.get("normal_output_color"))
                    text_cursor.insertText(output_message, text_format)
                    text_cursor.insertBlock()
            # 處理錯誤輸出
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    text_cursor = self.code_result.textCursor()
                    text_format = QTextCharFormat()
                    text_format.setForeground(actually_color_dict.get("error_output_color"))
                    text_cursor.insertText(error_message, text_format)
                    text_cursor.insertBlock()
        except queue.Empty:
            # 如果佇列是空的就忽略
            pass

        # 如果子程序已經結束(returncode 不為 None)，則完全退出
        if self.process is not None:
            if self.process.returncode == 0:
                self.full_exit_program()
            elif self.process.returncode is not None:
                self.full_exit_program()

            # 如果程式仍在執行，持續檢查狀態
            if self.still_run_program:
                # poll() 不會阻塞，只是更新 returncode
                self.process.poll()

    # 結束程式：將執行旗標設為 False，清理執行緒、佇列與子程序
    # Exit program: change run flag to false and clean read thread, queue, and process
    def exit_program(self) -> None:
        jeditor_logger.info("ExecManager exit_program")
        self.still_run_program = False
        # 等待讀取執行緒結束（設定 timeout 避免永久阻塞）
        # Wait for reader threads to finish (with timeout to avoid deadlock)
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread.join(timeout=2)
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread.join(timeout=2)
            self.read_program_error_output_from_thread = None
        # 清空佇列
        self.print_and_clear_queue()
        # 如果子程序存在，則終止並等待
        if self.process is not None:
            self.process.terminate()
            try:
                self.process.wait(timeout=5)
            except subprocess.TimeoutExpired:
                self.process.kill()
            if self.code_result is not None:
                text_cursor = self.code_result.textCursor()
                text_format = QTextCharFormat()
                text_format.setForeground(actually_color_dict.get("normal_output_color"))
                text_cursor.insertText(f"Program exit with code {self.process.returncode}", text_format)
                text_cursor.insertBlock()
            self.process = None

    # 清空輸出與錯誤佇列
    # Pull all remaining strings in queues and reset them
    def print_and_clear_queue(self) -> None:
        jeditor_logger.info("ExecManager print_and_clear_queue")
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    # 從子程序 stdout 持續讀取資料並放入輸出佇列
    # Continuously read from process stdout and put into output queue
    def read_program_output_from_process(self) -> None:
        jeditor_logger.info("ExecManager read_program_output_from_process")
        try:
            while self.still_run_program:
                program_output_data: str = self.process.stdout.readline(
                    self.program_buffer).decode(self.program_encoding, "replace")
                if not program_output_data and self.process.poll() is not None:
                    break
                if self.process:
                    self.process.stdout.flush()
                self.run_output_queue.put_nowait(program_output_data)
        except (OSError, ValueError):
            pass

    # 從子程序 stderr 持續讀取資料並放入錯誤佇列
    # Continuously read from process stderr and put into error queue
    def read_program_error_output_from_process(self) -> None:
        jeditor_logger.info("ExecManager read_program_error_output_from_process")
        try:
            while self.still_run_program:
                program_error_output_data: str = self.process.stderr.readline(
                    self.program_buffer).decode(self.program_encoding, "replace")
                if not program_error_output_data and self.process.poll() is not None:
                    break
                if self.process:
                    self.process.stderr.flush()
                self.run_error_queue.put_nowait(program_error_output_data)
        except (OSError, ValueError):
            pass



from PySide6.QtGui import QColor

# -----------------------------
# 基本語法規則設定 (數字、註解、字串)
# Basic syntax rule settings (numbers, comments, strings)
# -----------------------------
syntax_rule_setting_dict: dict = {
    "number_rule": {
        # 數字規則 (整數、十六進位、小數/科學記號)
        # Number rules (integer, hex, float/scientific notation)
        "rules": (
            r"\b[+-]?[0-9]+[lL]?\b",  # 整數 integer
            r"\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b",  # 十六進位 hex
            r"\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b"  # 浮點數/科學記號 float/scientific
        ),
        "color": QColor(0, 128, 255)  # 藍色 Blue
    },
    "comment_rule": {
        # 註解規則 (以 # 開頭直到換行)
        # Comment rule (starts with # until newline)
        "rules": (r"#[^\n]*",),
        "color": QColor(0, 230, 0)  # 綠色 Green
    },
    "string_rule": {
        # 字串規則 (單引號與雙引號)
        # String rules (single and double quotes)
        "rules": (
            r"'[^'\\]*(\\.[^'\\]*)*'",  # 單引號字串 single-quoted string
            r'"[^"\\]*(\\.[^"\\]*)*"',  # 雙引號字串 double-quoted string
        ),
        "color": QColor(0, 153, 0)  # 深綠色 Dark green
    }
}

# -----------------------------
# 關鍵字與內建函式設定
# Keywords and built-in functions
# -----------------------------
syntax_word_setting_dict: dict = {
    "keywords": {
        # Python 保留字 (語法關鍵字)
        # Python reserved keywords
        "words": (
            "False", "None", "True", "and", "as", "assert", "async",
            "await", "break", "class", "continue", "def", "del",
            "elif", "else", "except", "finally", "for", "from",
            "global", "if", "import", "in", "is", "lambda", "nonlocal",
            "not", "or", "pass", "raise", "return", "try", "while", "with", "yield"
        ),
        "color": QColor(255, 212, 102)  # 黃色 Yellow
    },
    "builtins_keyword": {
        # Python 內建函式與型別
        # Python built-in functions and types
        "words": (
            "abs", "aiter", "all", "any", "anext", "ascii",
            "bin", "bool", "breakpoint", "bytearray", "bytes",
            "callable", "chr", "classmethod", "compile", "complex",
            "delattr", "dict", "dir", "divmod",
            "enumerate", "eval", "exec",
            "filter", "float", "format", "frozenset",
            "getattr", "globals",
            "hasattr", "hash", "help", "hex",
            "id", "input", "int", "isinstance", "issubclass", "iter",
            "len", "list", "locals",
            "map", "max", "memoryview", "min",
            "next",
            "object", "oct", "open", "ord",
            "pow", "print", "property",
            "range", "repr", "reversed", "round",
            "set", "setattr", "slice", "sorted", "staticmethod",
            "str", "sum", "super",
            "tuple", "type",
            "vars",
            "zip",
            "__import__"
        ),
        "color": QColor(0, 255, 255)  # 青色 Cyan
    },
    "self": {
        # Python 類別中的 self 關鍵字
        # "self" keyword in Python classes
        "words": ("self",),
        "color": QColor(204, 0, 204)  # 紫色 Purple
    }
}

# -----------------------------
# 擴展語法設定 (其他語言可在此加入)
# Extended syntax settings (for other languages)
# -----------------------------
syntax_extend_setting_dict: dict = {
    # 目前為空，可依副檔名新增規則
    # Currently empty, can add rules for other file suffixes
}



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Union

from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QFontDatabase
from PySide6.QtWidgets import QWidget, QPlainTextEdit, QScrollArea, QLabel, QComboBox, QGridLayout, QPushButton, \
    QMessageBox, QSizePolicy, QLineEdit

from je_editor.pyside_ui.dialog.ai_dialog.set_ai_dialog import SetAIDialog
from je_editor.pyside_ui.main_ui.ai_widget.ai_config import AIConfig, ai_config
from je_editor.pyside_ui.main_ui.ai_widget.ask_thread import AskThread
from je_editor.pyside_ui.main_ui.ai_widget.langchain_interface import LangChainInterface
from je_editor.utils.json.json_file import read_json
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


class ChatUI(QWidget):

    def __init__(self, main_window: EditorMain):
        super().__init__()
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)  # 關閉視窗時自動釋放資源 / Auto delete on close
        self.main_window = main_window

        # ---------------- Chat Panel 聊天面板 ----------------
        self.chat_panel = QPlainTextEdit()  # 顯示聊天訊息的文字框 / Text area for chat messages
        self.chat_panel.setLineWrapMode(self.chat_panel.LineWrapMode.NoWrap)  # 不自動換行 / Disable line wrap
        self.chat_panel.setReadOnly(True)  # 設為唯讀，避免使用者直接輸入 / Read-only
        self.chat_panel_scroll_area = QScrollArea()  # 加入滾動區域 / Scroll area for chat panel
        self.chat_panel_scroll_area.setWidgetResizable(True)
        self.chat_panel_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.chat_panel_scroll_area.setWidget(self.chat_panel)
        self.chat_panel.setFont(QFontDatabase.font(self.font().family(), "", 16))  # 設定字體大小 / Set font size

        # ---------------- Prompt Input 輸入框 ----------------
        self.prompt_input = QLineEdit()  # 使用者輸入提示詞 / Input field for prompts
        self.prompt_input.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
        self.prompt_input.returnPressed.connect(self.call_ai_model)  # 按 Enter 時呼叫 AI / Call AI on Enter

        # ---------------- Font Size Combobox 字體大小選單 ----------------
        self.font_size_label = QLabel(language_wrapper.language_word_dict.get("font_size"))  # 標籤 / Label
        self.font_size_combobox = QComboBox()  # 下拉選單 / Dropdown for font size
        for font_size in range(2, 101, 2):  # 提供 2~100 的字體大小選項 / Font size options
            self.font_size_combobox.addItem(str(font_size))
        self.font_size_combobox.setCurrentText("16")  # 預設字體大小 / Default font size
        self.font_size_combobox.currentTextChanged.connect(self.update_panel_text_size)

        # ---------------- Buttons 按鈕 ----------------
        self.set_ai_config_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_set_ai_button"))
        self.set_ai_config_button.clicked.connect(self.set_ai_config)  # 開啟 AI 設定視窗 / Open AI config dialog

        self.load_ai_config_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_load_ai_button"))
        self.load_ai_config_button.clicked.connect(
            lambda: self.load_ai_config(show_load_complete=True))  # 載入設定 / Load config

        self.call_ai_model_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_call_ai_model_button"))
        self.call_ai_model_button.clicked.connect(self.call_ai_model)  # 呼叫 AI / Call AI

        # ---------------- Layout 版面配置 ----------------
        self.grid_layout = QGridLayout()
        self.grid_layout.addWidget(self.chat_panel_scroll_area, 0, 0, 1, 4)  # 聊天面板 / Chat panel
        self.grid_layout.addWidget(self.call_ai_model_button, 1, 0)  # 呼叫 AI 按鈕 / Call AI button
        self.grid_layout.addWidget(self.font_size_combobox, 1, 1)  # 字體大小選單 / Font size combobox
        self.grid_layout.addWidget(self.set_ai_config_button, 1, 2)  # 設定 AI 按鈕 / Set AI config button
        self.grid_layout.addWidget(self.load_ai_config_button, 1, 3)  # 載入設定按鈕 / Load AI config button
        self.grid_layout.addWidget(self.prompt_input, 2, 0, 1, 4)  # 輸入框 / Prompt input

        # ---------------- Variables 變數 ----------------
        self.ai_config: AIConfig = ai_config  # AI 設定物件 / AI config object
        self.lang_chain_interface: Union[LangChainInterface, None] = None  # LangChain 介面 / LangChain interface
        self.set_ai_config_dialog = None  # 設定對話框 / Config dialog

        # ---------------- Timer 計時器 ----------------
        self.pull_message_timer = QTimer(self)  # 定時檢查訊息佇列 / Timer to pull messages
        self.pull_message_timer.setInterval(1000)  # 每秒檢查一次 / Check every 1 second
        self.pull_message_timer.timeout.connect(self.pull_message)
        self.pull_message_timer.start()

        # ---------------- Set Layout 設定版面 ----------------
        self.setLayout(self.grid_layout)

        # ---------------- Load AI Config 載入 AI 設定 ----------------
        self.load_ai_config()

    # 更新聊天面板字體大小 / Update chat panel font size
    def update_panel_text_size(self):
        self.chat_panel.setFont(
            QFontDatabase.font(self.font().family(), "", int(self.font_size_combobox.currentText())))

    # 載入 AI 設定檔 / Load AI configuration file
    def load_ai_config(self, show_load_complete: bool = False):
        ai_config_file = Path.cwd() / ".jeditor" / "ai_config.json"
        if ai_config_file.exists():
            json_data: dict = read_json(str(ai_config_file))
            if json_data:
                # 確認 AI_model 設定存在且包含必要欄位 / Ensure AI_model config exists with required fields
                if json_data.get("AI_model") and isinstance(json_data.get("AI_model"), dict):
                    ai_info: dict = json_data.get("AI_model")
                    if ai_info.get("ai_base_url") and ai_info.get("chat_model"):
                        ai_config.choosable_ai.update(json_data)  # 更新全域設定 / Update global config
                    # 建立 LangChain 介面 / Initialize LangChain interface
                    self.lang_chain_interface = LangChainInterface(
                        main_window=self,
                        api_key=ai_info.get("ai_api_key"),
                        base_url=ai_info.get("ai_base_url"),
                        chat_model=ai_info.get("chat_model"),
                        prompt_template=ai_info.get("prompt_template"),
                    )
            if show_load_complete:
                load_complete = QMessageBox(self)
                load_complete.setWindowTitle(language_wrapper.language_word_dict.get("load_ai_messagebox_title"))
                load_complete.setText(language_wrapper.language_word_dict.get("load_ai_messagebox_text"))
                load_complete.exec()

    # 呼叫 AI 模型 / Call AI model
    def call_ai_model(self):
        if isinstance(self.lang_chain_interface, LangChainInterface):
            # 建立新執行緒處理 AI 請求 / Start a new thread for AI request
            # Store reference to prevent garbage collection before thread completes
            self._ask_thread = AskThread(lang_chain_interface=self.lang_chain_interface, prompt=self.prompt_input.text())
            self._ask_thread.start()
        else:
            # 若未正確設定 AI，顯示錯誤訊息 / Show error if AI not configured
            ai_info = ai_config.choosable_ai.get('AI_model', {})
            # Mask API key to prevent leaking credentials
            api_key = ai_info.get('ai_api_key', '')
            masked_key = f"{api_key[:4]}...{api_key[-4:]}" if api_key and len(api_key) > 8 else "(not set)"
            QMessageBox.warning(self,
                                language_wrapper.language_word_dict.get("call_ai_model_error_title"),
                                f"ai_api_key: {masked_key}, \n"
                                f"ai_base_url: {ai_info.get('ai_base_url')}, \n"
                                f"chat_model: {ai_info.get('chat_model')}, \n"
                                f"prompt_template: {ai_info.get('prompt_template')}")

    # 從訊息佇列中取出 AI 回覆並顯示 / Pull AI response from queue
    def pull_message(self):
        if not ai_config.message_queue.empty():
            ai_response = ai_config.message_queue.get_nowait()
            self.chat_panel.appendPlainText(ai_response)  # 顯示回覆 / Display response
            self.chat_panel.appendPlainText("\n")

    # 開啟 AI 設定對話框 / Open AI config dialog
    def set_ai_config(self):
        self.set_ai_config_dialog = SetAIDialog()
        self.set_ai_config_dialog.show()



from __future__ import annotations  # 啟用未來版本的型別註解功能 / Enable postponed evaluation of type annotations

from typing import TYPE_CHECKING  # 僅在型別檢查時使用，避免循環匯入 / Used only for type checking to avoid circular imports

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget  # 編輯器分頁元件 / Editor tab widget
from je_editor.utils.logging.loggin_instance import jeditor_logger  # 專案內的日誌紀錄器 / Project logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper  # 多語系支援 / Multi-language wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain  # 僅在型別檢查時匯入 / Import only for type checking

from PySide6.QtGui import QAction, QKeySequence  # Qt 動作與快捷鍵 / Qt actions and shortcuts
from yapf.yapflib.yapf_api import FormatCode  # YAPF 程式碼格式化工具 / YAPF code formatter

from je_editor.utils.json_format.json_process import reformat_json  # JSON 格式化工具 / JSON reformatter


def set_check_menu(ui_we_want_to_set: EditorMain) -> None:
    """
    建立「程式碼檢查/格式化」選單，並加入三個功能：
    - 使用 YAPF 重新格式化 Python 程式碼
    - 重新格式化 JSON
    - 檢查 Python 檔案格式

    Create "Check/Format Code" menu with three actions:
    - Reformat Python code with YAPF
    - Reformat JSON
    - Check Python file format
    """
    jeditor_logger.info(f"build_check_style_menu.py set_check_menu ui_we_want_to_set: {ui_we_want_to_set}")

    # 在主選單中新增一個子選單 / Add submenu to main menu
    ui_we_want_to_set.check_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("check_code_style_menu_label"))

    # === 1. Yapf Python 程式碼格式化 / Yapf Python code reformat ===
    ui_we_want_to_set.check_menu.yapf_check_python_action = QAction(
        language_wrapper.language_word_dict.get("yapf_reformat_label"))
    ui_we_want_to_set.check_menu.yapf_check_python_action.setShortcut(
        QKeySequence("Ctrl+Shift+Y"))  # 設定快捷鍵 / Set shortcut
    ui_we_want_to_set.check_menu.yapf_check_python_action.triggered.connect(
        lambda: yapf_check_python_code(ui_we_want_to_set)
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.yapf_check_python_action)

    # === 2. JSON 重新格式化 / Reformat JSON ===
    ui_we_want_to_set.check_menu.reformat_json_action = QAction(
        language_wrapper.language_word_dict.get("reformat_json_label"))
    ui_we_want_to_set.check_menu.reformat_json_action.setShortcut("Ctrl+j")
    ui_we_want_to_set.check_menu.reformat_json_action.triggered.connect(
        lambda: reformat_json_text(ui_we_want_to_set)
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.reformat_json_action)

    # === 3. Python 格式檢查 / Python format check ===
    ui_we_want_to_set.check_menu.check_python_format = QAction(
        language_wrapper.language_word_dict.get("python_format_checker"))
    ui_we_want_to_set.check_menu.check_python_format.setShortcut("Ctrl+Alt+p")
    ui_we_want_to_set.check_menu.check_python_format.triggered.connect(
        lambda: check_python_format(ui_we_want_to_set)
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.check_python_format)


def yapf_check_python_code(ui_we_want_to_set: EditorMain) -> None:
    """
    使用 YAPF 重新格式化目前分頁中的 Python 程式碼
    Reformat current tab's Python code using YAPF
    """
    jeditor_logger.info(f"build_check_style_menu.py yapf_check_python_code ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        code_text = widget.code_edit.toPlainText()  # 取得編輯器文字 / Get code text
        widget.code_result.setPlainText("")  # 清空結果區域 / Clear result area
        format_code = FormatCode(
            unformatted_source=code_text,
            style_config="google"  # 使用 Google 風格 / Use Google style
        )
        if isinstance(format_code, tuple):
            widget.code_edit.setPlainText(format_code[0])  # 將格式化後的程式碼寫回編輯器 / Write formatted code back


def reformat_json_text(ui_we_want_to_set: EditorMain) -> None:
    """
    重新格式化目前分頁中的 JSON 文字
    Reformat JSON text in the current editor tab
    """
    jeditor_logger.info(f"build_check_style_menu.py reformat_json_text ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        code_text = widget.code_edit.toPlainText()
        widget.code_result.setPlainText("")
        widget.code_edit.setPlainText(reformat_json(code_text))  # 呼叫 JSON 格式化工具 / Call JSON reformatter


def check_python_format(ui_we_want_to_set: EditorMain) -> None:
    """
    呼叫 EditorWidget 的檔案格式檢查功能
    Call EditorWidget's file format checker
    """
    jeditor_logger.info(f"build_check_style_menu.py check_python_format ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.check_file_format()



from PySide6.QtGui import QTextDocument, QAction
from PySide6.QtWidgets import QTextEdit

from je_editor.pyside_ui.dialog.search_ui.search_error_box import SearchResultBox
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger


class CodeRecord(QTextEdit):
    # 繼承自 QTextEdit，作為程式碼輸出紀錄區
    # Extend QTextEdit, used as a code output record area
    def __init__(self):
        jeditor_logger.info("Init CodeRecord")
        super().__init__()
        self.setLineWrapMode(self.LineWrapMode.NoWrap)  # 禁止自動換行 / disable line wrapping
        self.setReadOnly(True)  # 設為唯讀 / set as read-only

        # 建立搜尋錯誤的快捷動作
        # Create search error action with shortcut
        self.search_result_action = QAction("Search Error")
        self.search_result_action.setShortcut("Ctrl+e")
        self.search_result_action.triggered.connect(
            self.start_search_result_dialog  # 綁定觸發事件 / bind trigger event
        )
        self.addAction(self.search_result_action)

    def append(self, text: str) -> None:
        """
        新增文字到輸出區，若超過最大行數則清空
        Append text to output area, clear if exceeding max lines
        """
        jeditor_logger.info("CodeRecord append")
        max_line: int = user_setting_dict.get("max_line_of_output", 200000)
        if self.document().lineCount() >= max_line > 0:
            # 若行數超過設定，清空內容
            # Clear content if line count exceeds limit
            self.setPlainText("")
        super().append(text)

    def start_search_result_dialog(self) -> None:
        """
        開啟搜尋對話框，並綁定搜尋按鈕事件
        Open search dialog and bind search button events
        """
        jeditor_logger.info("CodeRecord start_search_result_dialog")
        self.search_result_box = SearchResultBox()  # 建立搜尋框 / create search box
        self.search_result_box.search_back_button.clicked.connect(
            self.find_back_text  # 綁定「上一個」搜尋 / bind "find previous"
        )
        self.search_result_box.search_next_button.clicked.connect(
            self.find_next_text  # 綁定「下一個」搜尋 / bind "find next"
        )
        self.search_result_box.show()  # 顯示搜尋框 / show search box

    def find_next_text(self) -> None:
        """
        搜尋下一個符合的文字
        Find next matching text
        """
        jeditor_logger.info("CodeRecord find_next_text")
        if self.search_result_box.isVisible():
            text = self.search_result_box.search_input.text()
            self.find(text)  # 使用 QTextEdit 內建 find 方法 / use QTextEdit built-in find

    def find_back_text(self) -> None:
        """
        搜尋上一個符合的文字
        Find previous matching text
        """
        jeditor_logger.info("CodeRecord find_back_text")
        if self.search_result_box.isVisible():
            text = self.search_result_box.search_input.text()
            # 使用 FindBackward 旗標往回搜尋
            # Use FindBackward flag to search backwards
            self.find(text, QTextDocument.FindFlag.FindBackward)



"""
default exception
"""
je_editor_error: str = "JEditor error"
"""
init exception
"""
je_editor_init_error: str = "JEditor failed to initialize"
je_editor_init_exec_manager_exception: str = "JEditor failed to initialize the program manager"
"""
exec exception
"""
je_editor_exec_error: str = "JEditor execution error"
file_not_fond_error: str = "File not found"
compiler_not_found_error: str = "Compiler not found"
"""
shell exception
"""
je_editor_shell_error: str = "JEditor shell execution error"
"""
file exception
"""
je_editor_save_file_error: str = "JEditor failed to save file"
je_editor_open_file_error: str = "JEditor failed to open file"
"""
json exception
"""
cant_reformat_json_error: str = "Cannot reformat JSON: is the type correct?"
wrong_json_data_error: str = "Failed to parse JSON"
cant_find_json_error: str = "Can't find JSON file"
cant_save_json_error: str = "Can't save JSON file"
"""
content data error
"""
content_set_compiler_error: str = "Error setting compiler using content"
je_editor_content_file_error: str = "JEditor content file error"



import json.decoder
import sys
from json import dumps
from json import loads

# 匯入自訂錯誤訊息與例外類別
# Import custom error messages and exception class
from je_editor.utils.exception.exception_tags import cant_reformat_json_error
from je_editor.utils.exception.exception_tags import wrong_json_data_error
from je_editor.utils.exception.exceptions import JEditorJsonException
from je_editor.utils.logging.loggin_instance import jeditor_logger


def __process_json(json_string: str, **kwargs) -> str:
    """
    功能說明 (Function Description):
    嘗試將輸入的 JSON 字串重新格式化 (pretty print)。
    Try to reformat the input JSON string (pretty print).

    :param json_string: JSON 格式字串 / JSON formatted string
    :param kwargs: 額外參數傳給 json.dumps / extra arguments for json.dumps
    :return: 格式化後的 JSON 字串 / formatted JSON string
    """
    try:
        # 嘗試先將字串解析為 JSON，再重新輸出為縮排格式
        # Try to parse string into JSON, then dump with indentation
        return dumps(loads(json_string), indent=4, sort_keys=True, **kwargs)
    except json.JSONDecodeError as error:
        # 如果 JSON 格式錯誤，輸出錯誤訊息到 stderr 並拋出例外
        # If JSON format is invalid, print error to stderr and raise exception
        print(wrong_json_data_error, file=sys.stderr)
        raise error
    except TypeError:
        # 如果輸入不是合法 JSON 字串，嘗試直接將物件轉為 JSON
        # If input is not a valid JSON string, try dumping the object directly
        try:
            return dumps(json_string, indent=4, sort_keys=True, **kwargs)
        except TypeError:
            # 若仍失敗，拋出自訂例外
            # If still fails, raise custom exception
            raise JEditorJsonException(wrong_json_data_error)


def reformat_json(json_string: str, **kwargs) -> str:
    """
    功能說明 (Function Description):
    對外提供的 JSON 格式化函式，會呼叫內部的 __process_json。
    Public function to reformat JSON string, calls __process_json internally.

    :param json_string: JSON 格式字串 / JSON formatted string
    :param kwargs: 額外參數傳給 json.dumps / extra arguments for json.dumps
    :return: 格式化後的 JSON 字串 / formatted JSON string
    """
    # 記錄日誌，方便除錯與追蹤
    # Log the input string and kwargs for debugging and tracking
    jeditor_logger.info(f"json_process.py reformat_json "
                        f"json_string: {json_string} "
                        f"kwargs: {kwargs}")
    try:
        return __process_json(json_string, **kwargs)
    except JEditorJsonException:
        # 捕捉自訂例外並重新拋出
        # Catch custom exception and re-raise
        raise JEditorJsonException(cant_reformat_json_error)
